#=========================================================================================
#@(#)Thierry: $Id$
#
#    Handle interactive action definition 
#
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Thierry CHEVALIER <t_chevalier@libertysurf.fr>
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 

from string import *

from Tkinter import *
from FileDialog import *
from tkSimpleDialog import *

import Pmw

from lib.ActionDef import *

from PortsEditor import *
from ParametersEditor import *
from ServersEditor import *

class ActionEditor :
	"""Handle interactive action definition"""

	# Generic methods
	#--------------------------------------------
	def __init__(self, top):
		self.top= top
		self._initGUI(top)

	# Public methods
	#--------------------------------------------
	def Clean(self):
		self.vname.set("")
		self.vdesc.set("")
		self.inputs.Clean()
		self.outputs.Clean()
		self.parameters.Clean()
		self.servers.Clean()

	def Load(self,fid):
		x= ActionDefXMLParser()
		while 1:
			l= fid.readline()
			if l=="":
				break
			x.feed(l)
		for a in x.actions.keys():
			print "Setting action ",a
			self.SetActionDef(x.actions[a])
		x.close()
		del x

	def Save(self,fid,indent):
		actiondef=self.GetActionDef()
		actiondef.Save(fid,indent)
		del actiondef

	def SetActionDef(self,actiondef):
		self.Clean()
		self.vname.set(actiondef.name)
		self.vdesc.set(actiondef.desc)
		for inputport in actiondef.inputorder:
			p= self.inputs.AddPort()
			p.SetPortDef(actiondef.inputPorts[inputport])
			AddPortType(actiondef.inputPorts[inputport].type)
		for outputport in actiondef.outputorder:
			p= self.outputs.AddPort()
			p.SetPortDef(actiondef.outputPorts[outputport])
			AddPortType(actiondef.outputPorts[outputport].type)
		for parameter in actiondef.paramorder:
			p= self.parameters.AddParameter()
			p.SetParameterDef(actiondef.parameters[parameter])
		for server in actiondef.servers.keys():
			s= self.servers.AddServer()
			s.SetServerDef(actiondef.servers[server])

	def GetActionDef(self):
		name= self.vname.get()
		desc= self.vdesc.get()

		a= ActionDef(name,"action",desc)

		inputports= self.inputs.ports.keys()
		inputports.sort()
		for inputport in inputports:
			a.AddInputPort(self.inputs.ports[inputport].GetPortDef())

		outputports= self.outputs.ports.keys()
		outputports.sort()
		for outputport in outputports:
			a.AddOutputPort(self.outputs.ports[outputport].GetPortDef())

		parameters= self.parameters.parameters.keys()
		parameters.sort()
		for parameter in parameters:
			a.AddParameter(self.parameters.parameters[parameter].GetParameterDef())

		servers= self.servers.servers.keys()
		servers.sort()
		for server in servers:
			a.AddServer(self.servers.servers[server].GetServerDef())

		return a

	# Internal GUI methods
	#--------------------------------------------
	def _load(self):
		file= LoadFileDialog(self.top).go(pattern="*.action.xml")
		if file is None:
			print "Unable to load action, no filename provided"
		else:
			print "=================== Action reading for ", file, " =================="
			fid=open(file,'r')
			self.Load(fid)
			fid.close()
			print "=============== End of action reading for ", file, " ==============="

	def _save(self):
		file= SaveFileDialog(self.top).go(pattern="*.action.xml")
		if file is None:
			print "Unable to save map, no filename provided"
		else:
			print "=================== Action writing for ", file, " =================="
			fid=open(file,'w')
			self.Save(fid,"")
			fid.close()
			print "=============== End of action writing for ", file, " ==============="

	def _initGUI(self,top):
		self.wf= Frame(top, relief="raised")
		self.wf.pack(side="top", fill="both", expand="yes")
		self.balloon = Pmw.Balloon(self.wf)

		# Action name
		f=Frame(self.wf)
		Label(f,text="Action name").pack(side="left")
		self.vname= StringVar()
		self.wname=Entry(f,textvariable=self.vname)
		self.wname.pack(side="right",fill="x",expand="yes")
		self.balloon.bind(self.wname, 'Name of this action')
		f.pack(side="top", fill="x", expand="no", padx="5", pady="5")
		
		# Action desc
		f=Frame(self.wf)
		Label(f,text="Description").pack(side="left")
		self.vdesc= StringVar()
		e=Entry(f,textvariable=self.vdesc)
		e.pack(side="right",fill="x",expand="yes")
		self.balloon.bind(e, 'Comment associated to this action')
		f.pack(side="top", fill="x", expand="no", padx="5", pady="5")
		
		# Action details
		self.wnb= Pmw.NoteBook(self.wf, hull_width= 400, hull_height= 300)
		self.wnb.pack(side="top",fill="both", expand="yes")
		self.winputs 	= self.wnb.add('InputPorts')
		self.inputs		= InputPortsEditor(self.winputs)
		self.woutputs 	= self.wnb.add('OutputPorts')
		self.outputs	= OutputPortsEditor(self.woutputs)
		self.wparameters= self.wnb.add('Parameters')
		self.parameters	= ParametersEditor(self.wparameters)
		self.wservers 	= self.wnb.add('Servers')
		self.servers	= ServersEditor(self.wservers)

		f=Frame(self.wf)
		Button(f,text="Load action",command=self._load).pack(side="right")
		Button(f,text="Save action",command=self._save).pack(side="right")
		f.pack(side="bottom",expand="no")

