#=========================================================================================
#@(#)Thierry: $Id$
#
#    Handle interactive parameter definition 
#
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Thierry CHEVALIER <t_chevalier@libertysurf.fr>
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 

from string import *

from Tkinter import *
import Pmw

from lib.ParameterDef import *

class ParameterEditor :
	"""Handle interactive parameter definition"""

	_bgcolor= "#5555ff"		# ParameterEditor background color

	# Generic methods
	#--------------------------------------------
	def __init__(self, parameterseditor, top, id):
		self.parameterseditor= parameterseditor
		self.top= top
		self.id= id
		self._initGUI(top)

	# Public methods
	#--------------------------------------------
	def Delete(self):
		"""Delete parameter"""
		self.parameterseditor._delparameter(self.id)
		self._delGUI()
		
	def SetParameterDef(self,parameterdef):
		self.vname.set(parameterdef.name)
		self.vdesc.set(parameterdef.desc)
		self.vtype.set(parameterdef.type)

	def GetParameterDef(self):
		name= self.vname.get()
		desc= self.vdesc.get()
		type= self.vtype.get()
		p= ParameterDef(name,desc,type)
		return p

	# Internal GUI methods
	#--------------------------------------------
	def _setparametertype(self,e):
		popup=Menu(self.top)
		popup.add_command(label="string",command=lambda tkvar=self.vtype, type="string": tkvar.set(type))
		popup.add_command(label="float",command=lambda tkvar=self.vtype, type="float": tkvar.set(type))
		popup.add_command(label="integer",command=lambda tkvar=self.vtype, type="integer": tkvar.set(type))
		popup.post(e.x_root,e.y_root)
		popup.grab_set()
		popup.mainloop()

	def _delGUI(self):
		self.wf.destroy()

	def _initGUI(self,top):
		self.wf= Frame(top, relief="raised",background=self._bgcolor)
		self.wf.pack(side="top", fill="both", expand="yes", padx="2", pady="2")
		self.balloon = Pmw.Balloon(self.wf)

		f=Frame(self.wf,background=self._bgcolor)
		Label(f,text="Name",background=self._bgcolor).pack(side="left")
		self.vname= StringVar()
		e=Entry(f,textvariable=self.vname)
		e.pack(side="left",fill="x",expand="yes")
		self.balloon.bind(e, 'Enter parameter name')
		Label(f,text="Type",background=self._bgcolor).pack(side="left")
		self.vtype= StringVar()
		self.vtype.set("string")
		l=Label(f,textvariable=self.vtype,relief="raised")
		l.pack(side="left",fill="x",expand="yes")
		l.bind("<ButtonPress-3>", self._setparametertype)
		self.balloon.bind(l, 'Select type among available ones using right mouse key')
		b=Button(f,text="Delete",command=self.Delete)
		b.pack(side="right", expand="no",  padx="5", pady="5")
		self.balloon.bind(b, 'Delete this parameter')
		f.pack(side="top",fill="x",expand="yes", padx="5", pady="5")
		
		f=Frame(self.wf,background=self._bgcolor)
		l=Label(f,text="Description",background=self._bgcolor)
		l.pack(side="left")
		self.vdesc= StringVar()
		e=Entry(f,textvariable=self.vdesc)
		e.pack(side="left",fill="x",expand="yes",  padx="5", pady="5")
		self.balloon.bind(e, 'Comment associated to this parameter')
		f.pack(side="bottom",fill="x",expand="yes", padx="5", pady="5")

