#=========================================================================================
#@(#)Thierry: $Id$
#
#    Handle interactive port definition 
#
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Thierry CHEVALIER <t_chevalier@libertysurf.fr>
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 

from string import *

from Tkinter import *
import Pmw

from lib.PortDef import *

from PortTypeEditor import *

class PortEditor :
	"""Handle interactive port definition"""

	_bgcolor= "#5555ff"		# PortEditor background color

	# Generic methods
	#--------------------------------------------
	def __init__(self, portseditor, top, id):
		self.portseditor= portseditor
		self.top= top
		self.id= id
		self._initGUI(top)

	# Public methods
	#--------------------------------------------
	def Delete(self):
		"""Delete port"""
		self.portseditor._delport(self.id)
		self._delGUI()

	def SetPortDef(self,portdef):
		self.vname.set(portdef.name)
		self.vdesc.set(portdef.desc)
		self.vtype.set(portdef.type)

	# Internal GUI methods
	#--------------------------------------------
	def _setporttype(self,e):
		SetPortType(self.wf,e,self.vtype)

	def _delGUI(self):
		self.wf.destroy()

	def _initGUI(self,top):
		self.wf= Frame(top, relief="raised",background=self._bgcolor)
		self.wf.pack(side="top", fill="both", expand="yes", padx="2", pady="2")
		self.balloon = Pmw.Balloon(self.wf)

		f=Frame(self.wf,background=self._bgcolor)
		Label(f,text="Name",background=self._bgcolor).pack(side="left")
		self.vname= StringVar()
		e=Entry(f,textvariable=self.vname)
		e.pack(side="left",fill="x",expand="yes")
		self.balloon.bind(e, 'Enter port name')
		Label(f,text="Type",background=self._bgcolor).pack(side="left")
		self.vtype= StringVar()
		self.vtype.set("<none>")
		l=Label(f,textvariable=self.vtype,relief="raised")
		l.pack(side="left",fill="x",expand="yes")
		l.bind("<ButtonPress-3>", self._setporttype)
		self.balloon.bind(l, 'Select type among available ones using right mouse key')
		b=Button(f,text="Delete",command=self.Delete)
		b.pack(side="right", expand="no",  padx="5", pady="5")
		self.balloon.bind(b, 'Delete this port')
		f.pack(side="top",fill="x",expand="yes", padx="5", pady="5")
		
		f=Frame(self.wf,background=self._bgcolor)
		l=Label(f,text="Description",background=self._bgcolor)
		l.pack(side="left")
		self.vdesc= StringVar()
		e=Entry(f,textvariable=self.vdesc)
		e.pack(side="left",fill="x",expand="yes",  padx="5", pady="5")
		self.balloon.bind(e, 'Comment associated to this port')
		f.pack(side="bottom",fill="x",expand="yes", padx="5", pady="5")


class InputPortEditor(PortEditor):
	def GetPortDef(self):
		name= self.vname.get()
		desc= self.vdesc.get()
		type= self.vtype.get()
		category= "object"
		p= InputPortDef(name,desc,type,category)
		return p

class OutputPortEditor(PortEditor):
	def GetPortDef(self):
		name= self.vname.get()
		desc= self.vdesc.get()
		type= self.vtype.get()
		category= "object"
		p= OutputPortDef(name,desc,type,category)
		return p

