#=========================================================================================
#@(#)Thierry: $Id$
#
#    Handle interactive factory definition 
#
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Thierry CHEVALIER <t_chevalier@libertysurf.fr>
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 

from string import *

from Tkinter import *
from FileDialog import *
from tkSimpleDialog import *

import Pmw

from lib.FactoryDef import *

from ActionEditor.ActionEditor import *
from ActionEditor.PortTypeEditor import *

import FactoryGenerator.Generation

class FactoryEditor :
	"""Handle interactive factory definition"""

	# Generic methods
	#--------------------------------------------
	def __init__(self, top):
		self.top= top
		self.actions={}
		self._initGUI(top)

	# Public methods
	#--------------------------------------------
	def Load(self,fid):
		x= FactoryDefXMLParser()
		while 1:
			l= fid.readline()
			if l=="":
				break
			x.feed(l)
		for f in x.factories.keys():
			print "Setting factory ",f
			self.SetFactoryDef(x.factories[f])
		x.close()
		del x

	def Save(self,fid,indent):
		factorydef=self.GetFactoryDef()
		factorydef.Save(fid,indent)
		del factorydef

	def SetFactoryDef(self,factorydef):
		self.vname.set(factorydef.name)
		self.vdesc.set(factorydef.desc)
		for name in factorydef.actions.keys():
			self.AddAction(name)
			self.actions[name].SetActionDef(factorydef.actions[name])

	def GetFactoryDef(self):
		name= self.vname.get()
		desc= self.vdesc.get()
		f=FactoryDef(name,desc)
		for a in self.actions.keys():
			f.AddAction(self.actions[a].GetActionDef())
		return f

	def AddAction(self,name):
		w= self.wnb.add(name)
		self.actions[name]= ActionEditor(w)
		self.actions[name].vname.set(name)
		self.actions[name].wname.configure(state="disabled")

	def DeleteAction(self,name):
		del self.actions[name]
		self.wnb.delete(name)

	# Internal GUI methods
	#--------------------------------------------
	def _addaction(self,result):
		if result is None or result == 'Cancel':
			self._addactiondialog.deactivate(result)
		else:
			name=self._addactiondialog.get()
			self.AddAction(name)
			self._addactiondialog.deactivate()

	def _deleteaction(self):
		name= self.wnb.getcurselection()
		if name == None:
			print "No actions left to delete !"
		else:
			self.deleteactionconfirm = Pmw.MessageDialog(self.wf,
										title = name,
										message_text = 'Really delete ?',
										defaultbutton = 0,
										buttons = ('OK', 'Cancel'))
			confirm=self.deleteactionconfirm.activate()
			if confirm == 'OK':
				self.DeleteAction(name)

	def _load(self):
		file= LoadFileDialog(self.top).go(pattern="*.factory.xml")
		if file is None:
			print "Unable to load factory, no filename provided"
		else:
			print "=================== Factory reading for ", file, " =================="
			fid=open(file,'r')
			self.Load(fid)
			fid.close()
			print "=============== End of factory reading for ", file, " ==============="

	def _save(self):
		file= SaveFileDialog(self.top).go(pattern="*.factory.xml")
		if file is None:
			print "Unable to save factory, no filename provided"
		else:
			print "=================== Factory writing for ", file, " =================="
			fid=open(file,'w')
			self.Save(fid,"")
			fid.close()
			print "=============== End of factory writing for ", file, " ==============="

	def _generate(self):
		dirname= SaveFileDialog(self.top).go(pattern="*")
		if dirname is None:
			print "Unable generate, no directory name provided"
		else:
			factorydef=self.GetFactoryDef()
			FactoryGenerator.Generation.Generate(factorydef,dirname)
			del factorydef
		

	def _initGUI(self,top):
		self.wf= Frame(top, relief="raised")
		self.wf.pack(side="top", fill="both", expand="yes")
		self.balloon = Pmw.Balloon(self.wf)

		# Factory name
		f=Frame(self.wf)
		Label(f,text="Factory name").pack(side="left")
		self.vname= StringVar()
		e=Entry(f,textvariable=self.vname)
		e.pack(side="right",fill="x",expand="yes")
		self.balloon.bind(e, 'Name of this action')
		f.pack(side="top", fill="x", expand="no", padx="5", pady="5")
		
		# Factory desc
		f=Frame(self.wf)
		Label(f,text="Description").pack(side="left")
		self.vdesc= StringVar()
		e=Entry(f,textvariable=self.vdesc)
		e.pack(side="right",fill="x",expand="yes")
		self.balloon.bind(e, 'Comment associated to this action')
		f.pack(side="top", fill="x", expand="no", padx="5", pady="5")
		
		# Actions
		self.wnb= Pmw.NoteBook(self.wf, hull_width= 400, hull_height= 500)
		self.wnb.pack(side="top",fill="both", expand="yes")

		# Add/Delete actions, port types
		self._addactiondialog = Pmw.PromptDialog(self.wf,
										title = 'Add a new action',
										label_text = 'New action name:',
										entryfield_labelpos = 'n',
										defaultbutton = 0,
										buttons = ('OK', 'Cancel'),
										command = self._addaction)
		self._addactiondialog.withdraw()
		f=Frame(self.wf)
		Button(f,text="Add action",command=self._addactiondialog.activate).pack(side="left",expand="no")
		Button(f,text="Delete action",command=self._deleteaction).pack(side="left",expand="no")
		PortTypeEditor(f)
		f.pack(side="top",fill="x",expand="no")

		# Load/Save/Generate
		f=Frame(self.wf)
		Button(f,text="Load factory",command=self._load).pack(side="right",expand="no")
		Button(f,text="Save factory",command=self._save).pack(side="right",expand="no")
		self.wgenerate= Button(f,text="Generate C++",command=self._generate)
		self.wgenerate.pack(side="right",expand="no")
		f.pack(side="bottom",fill="x",expand="no")


