#=========================================================================================
#@(#)Thierry: $Id$
#
#   Handle factory code generation
#
#-----------------------------------------------------------------------------------------
# 	Initial version of this code, is a translation of the Tcl Action Generation
# 	code, written by P.Lafraise and available in the ActionFactory core
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Dassault Aviation
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 

import sys, os, shutil, time

from lib.PortDef import *
from lib.ParameterDef import *
from lib.ActionDef import *
from lib.FactoryDef import *

class ActionGeneration:
	_version= "1.0"
	_date	= "07/11/2000"
	_comment= "Action Code Generation for ActionFactory Corba Server"
	_files	= { 	'cfg_dir' 	  :	"config",
			'cfg_suff'	  :	"config",
			'idl_dir'	  :	"idl",
			'idl_suff'	  :	"idl",
			'bin_dir'	  :	"bin",
			'lib_dir'	  :	"lib",
			'impl_suff'	  :	"_impl",
			'ptr_suff'	  :	"_ptr",
			'var_suff'	  :	"_var",
			'impl_header_dir' :	"include",
			'impl_header_suff':	"_impl.h",
			'impl_src_dir'	  :	"src",
			'impl_src_suff'	  :	"_impl.C",
			'main_src_suff'	  :	"_srv_ORBACUS.C",
			'mod_header_dir'  :	"include",
			'mod_header_suff' :	"_mod.h",
			'test_dir'	  :	"test",
			'doc_dir'	  :	"doc"
		  }

	def __init__(self,factorydef,dirname,backup=0):
		self.factorydef= factorydef
		self.dirname= dirname
		self.backup= backup


	# Public methods
	#-------------------------

	def CreateDirs(self):
		"""Create directories"""
		print "Generating directory structure..."
		self._createDir(self.dirname)
		self._createDir(self.dirname + os.sep + self._files['idl_dir'])
		self._createDir(self.dirname + os.sep + self._files['impl_header_dir'])
		self._createDir(self.dirname + os.sep + self._files['impl_src_dir'])
		self._createDir(self.dirname + os.sep + self._files['lib_dir'])
		self._createDir(self.dirname + os.sep + self._files['bin_dir'])
		self._createDir(self.dirname + os.sep + self._files['test_dir'])
		self._createDir(self.dirname + os.sep + self._files['doc_dir'])
		self._createDir(self.dirname + os.sep + self._files['doc_dir'] + os.sep + "html")
		self._createDir(self.dirname + os.sep + self._files['doc_dir'] + os.sep + "html" + os.sep + "RefGuide")
		


	def GenerateActions(self):
		"""Generate actions..."""
		print "Generate actions..."
		for actionname in self.factorydef.actions.keys():
			print "\t",actionname,"\t: ",
			action= self.factorydef.actions[actionname]
			
			# Generate idl
			print " IDL",
			path= self.dirname + os.sep + self._files['idl_dir']
			name= "Action" + action.name + "." + self._files['idl_suff']
			filename= path + os.sep + name
			self._checkbackup(filename)
			fid=open(filename,'w')
			self._generateActionIDL(action,fid)
			fid.close()	
			
			# Generate header
			print ", header",
			path= self.dirname + os.sep + self._files['impl_header_dir']
			name= "Action" + action.name + self._files['impl_header_suff']
			filename= path + os.sep + name
			self._checkbackup(filename)
			fid=open(filename,'w')
			self._generateActionHeader(action,fid)
			fid.close()	
			
			# Generate source
			print ", source",
			path= self.dirname + os.sep + self._files['impl_src_dir']
			name= "Action" + action.name + self._files['impl_src_suff']
			filename= path + os.sep + name
			self._checkbackup(filename)
			fid=open(filename,'w')
			self._generateActionSource(action,fid)
			fid.close()
			
			print " ...OK !"
			

			
	def GenerateFactory(self):
		"""Generate factory..."""
		print "Generate factory..."
		print "\t",self.factorydef.name,"\t: ",
			
		# Generate idl
		print " IDL",
		path= self.dirname + os.sep + self._files['idl_dir']
		name= "FactoryAction" + self.factorydef.name + "." + self._files['idl_suff']
		filename= path + os.sep + name
		self._checkbackup(filename)
		fid=open(filename,'w')
		self._generateFactoryIDL(self.factorydef,fid)
		fid.close()	
		
		# Generate header
		print ", header",
		path= self.dirname + os.sep + self._files['impl_header_dir']
		name= "FactoryAction" + self.factorydef.name + self._files['impl_header_suff']
		filename= path + os.sep + name
		self._checkbackup(filename)
		fid=open(filename,'w')
		self._generateFactoryHeader(self.factorydef,fid)
		fid.close()	
		
		# Generate source
		print ", source",
		path= self.dirname + os.sep + self._files['impl_src_dir']
		name= "FactoryAction" + self.factorydef.name + self._files['impl_src_suff']
		filename= path + os.sep + name
		self._checkbackup(filename)
		fid=open(filename,'w')
		self._generateFactorySource(self.factorydef,fid)
		fid.close()
		
		# Generate server
		print ", server",
		path= self.dirname + os.sep + self._files['impl_src_dir']
		name= "FactoryAction" + self.factorydef.name + self._files['main_src_suff']
		filename= path + os.sep + name
		self._checkbackup(filename)
		fid=open(filename,'w')
		self._generateFactoryServer(self.factorydef,fid)
		fid.close()
		
		print " ...OK !"
		
	
	def GenerateMakefiles(self):
		"""Generate makefiles..."""
		print "Generate makefiles..."
			
		# Generate idl
		print "\tIDLs makefiles",
		path= self.dirname + os.sep + self._files['idl_dir']
		name= "Makefile"
		filename= path + os.sep + name
		self._checkbackup(filename)
		fid=open(filename,'w')
		self._generateMakefileIDL(self.factorydef,fid)
		fid.close()
		
		# Generate header
		print ", headers makefile",
		path= self.dirname + os.sep + self._files['impl_header_dir']
		name= "Makefile"
		filename= path + os.sep + name
		self._checkbackup(filename)
		fid=open(filename,'w')
		self._generateMakefileHeader(self.factorydef,fid)
		fid.close()
		
		# Generate source
		print ", sources makefile",
		path= self.dirname + os.sep + self._files['impl_src_dir']
		name= "Makefile"
		filename= path + os.sep + name
		self._checkbackup(filename)
		fid=open(filename,'w')
		self._generateMakefileSource(self.factorydef,fid)
		fid.close()
		
		print " ...OK !"
		print "\tMain makefile has to be hand generated because it's too dependant of your environment"
		
		
	# Private methods
	#-------------------------
	
	def _generateMakefileGenericHeader(self,fid):
		date=time.asctime(time.localtime(time.time()))
		fid.write("#================================================================\n")
		fid.write("# File Generated by ActionGeneration, version %s.\n"%(self._version))
		fid.write("# DO NOT EDIT\n")
		fid.write("# Created on %s\n"%(date))
		fid.write("#\n")
		fid.write("# (C) 2000 by Dassault Aviation\n")
		fid.write("#----------------------------------------------------------------\n")

	def _generateGenericHeader(self,fid):
		date=time.asctime(time.localtime(time.time()))
		fid.write("/**********************************************************************\n")
		fid.write("\n")
		fid.write("\tFile Generated by ActionGeneration, version %s.\n"%(self._version))
		fid.write("\tDO NOT EDIT\n")
		fid.write("\tCreated on %s\n"%(date))
		fid.write("\n")
		fid.write("\t(C) 2000 by Dassault Aviation\n")
		fid.write("\n")
		fid.write("**********************************************************************/\n")
		
		
		
	def _generateMakefileIDL(self,factorydef,fid):
		self._generateMakefileGenericHeader(fid)
		fid.write("include $(STD_RULES_DIR)/make.idl.top\n")
		fid.write("\n")
		fid.write("#================================================================\n")
		fid.write("# Definitions\n")
		fid.write("#----------------------------------------------------------------\n")
		fid.write("\n")
		fid.write("OBJECT0= FactoryAction%s\n"%(factorydef.name))
		i=1
		for actionname in factorydef.actions.keys():
			fid.write("OBJECT%s= Action%s\n"%(i,actionname))
			i= i+1
		fid.write("\n")
		fid.write("\n")
		fid.write("OBJECTS= \\\n $(OBJECT0)")
		i=1
		for actionname in factorydef.actions.keys():
			fid.write("\\\n $(OBJECT%d) "%(i))
			i= i+1
		fid.write("\n")
		fid.write("\n")
		fid.write("OBJECTS_HTML= \\\n $(OBJECT0)$(ORB_HTML_SUFF) ")
		i=1
		for actionname in factorydef.actions.keys():
			fid.write("\\\n $(OBJECT%d)$(ORB_HTML_SUFF) "%(i))
			i= i+1
		fid.write("\n")
		fid.write("\n")
		fid.write("OBJECTS_IDL= \\\n $(OBJECT0)$(ORB_IDL_SUFF) ")
		i=1
		for actionname in factorydef.actions.keys():
			fid.write("\\\n $(OBJECT%d)$(ORB_IDL_SUFF) "%(i))
			i= i+1
		fid.write("\n")
		fid.write("\n")
		fid.write("OBJECTS_GENE= \\\n $(OBJECT0)$(ORB_GENE_SUFF) ")
		i=1
		for actionname in factorydef.actions.keys():
			fid.write("\\\n $(OBJECT%d)$(ORB_GENE_SUFF) "%(i))
			i= i+1
		fid.write("\n")
		fid.write("\n")

		fid.write("include $(STD_RULES_DIR)/make.idl.bottom\n")

		
		
		
	def _generateFactoryIDL(self,factorydef,fid):
		nb_actions	= len(factorydef.actions)
		nb_servers	= len(factorydef.GetServertypeList())
  
		self._generateGenericHeader(fid)
		fid.write("/**********************************************************************\n")
		fid.write("\n")
		fid.write("Application Action Factory : FactoryAction%s.%s\n"%(factorydef.name,self._files['idl_suff']))
		fid.write("\n")
		fid.write("Description : FactoryAction%s IDL\n"%(factorydef.name))
		fid.write("\n")
		fid.write("**********************************************************************/\n")
		fid.write("\n")
		fid.write("#ifndef FactoryAction%s_idl\n"%(factorydef.name))
		fid.write("#define FactoryAction%s_idl\n"%(factorydef.name))
		fid.write("\n")
		fid.write("#include <FactoryAction.idl>\n")
		fid.write("\n")
		
		# Action list
		fid.write("enum ENUM_ACTION {\n")
		i=0
		for actionname in factorydef.actions.keys():
			i= i+1
			if i == nb_actions:
				fid.write("\t%s\n};\n"%(actionname))
			else:
				fid.write("\t%s,\n"%(actionname))
		fid.write("\n")

		# Used FactoryObjects
		fid.write("enum ENUM_FACT {\n")
		i=0
		for servertype in factorydef.GetServertypeList():
			i= i+1
			if i == nb_servers:
				fid.write("\tSERVER_%s\n};\n"%(servertype))
			else:
				fid.write("\tSERVER_%s,\n"%(servertype))
		fid.write("\n")
		

		# Interface definition
		fid.write("interface FactoryAction%s : FactoryAction {\n"%(factorydef.name))
		fid.write("};\n")
		fid.write("\n")
		
		fid.write("#endif\n")
		
		
		
	def _generateActionIDL(self,actiondef,fid):
		self._generateGenericHeader(fid)
		fid.write("/**********************************************************************\n")
		fid.write("\n")
		fid.write("Application Action Factory : Action%s.%s\n"%(actiondef.name,self._files['idl_suff']))
		fid.write("\n")
		fid.write("Description : Action%s IDL\n"%(actiondef.name))
		fid.write("\n")
		fid.write("**********************************************************************/\n")
		fid.write("\n")
		fid.write("#ifndef Action%s_idl\n"%(actiondef.name))
		fid.write("#define Action%s_idl\n"%(actiondef.name))
		fid.write("\n")
		fid.write("#include <Action.idl>\n")
		fid.write("\n")
		fid.write("interface Action%s : Action\n"%(actiondef.name))
		fid.write("{\n")
		fid.write("};\n")
		fid.write("#endif\n")
		
		
				
	def _generateMakefileHeader(self,factorydef,fid):
		self._generateMakefileGenericHeader(fid)
		fid.write("include $(STD_RULES_DIR)/make.include.top\n")
		fid.write("\n")
		fid.write("#=======================================\n")
		fid.write("# Definitions\n")
		fid.write("#---------------------------------------\n")
		fid.write("HEADERS=\\\n FactoryAction%s$(ORB_IMPL_SUFF).h"%(factorydef.name))
		for actionname in factorydef.actions.keys():
			fid.write("\\\n Action%s$(ORB_IMPL_SUFF).h"%(actionname))
		fid.write("\n")
		fid.write("\n")
		fid.write("include $(STD_RULES_DIR)/make.include.bottom\n")
		
		
		
	def _generateFactoryHeader(self,factorydef,fid):
		nb_actions	= len(factorydef.actions)
		nb_servers	= len(factorydef.GetServertypeList())
  
		# Titles and Commentary
		#----------------------
		self._generateGenericHeader(fid)
		fid.write("/**********************************************************************\n")
		fid.write("\n")
		fid.write("Application Action Factory : FactoryAction%s%s.h\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("Description : FactoryAction%s header implementation\n"%(factorydef.name))
		fid.write("\n")
		fid.write("**********************************************************************/\n")
		fid.write("\n")
		fid.write("#ifndef FactoryAction%s%s_h\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("#define FactoryAction%s%s_h\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\n")
		

		# Includes
		#----------
		fid.write("#include \"Corba_def.h\"\n")
		fid.write("#include \"FactoryAction_impl.h\"\n")
		for actionname in factorydef.actions.keys():
			fid.write("#include \"Action%s%s.h\"\n"%(actionname,self._files['impl_suff']))
		fid.write("#include \"GeneralUtils.h\"\n")
		fid.write("\n")

		# ORBacus specific includes
		fid.write("#if defined(ORBACUS)\n")
		fid.write("#include <OB/CORBA.h>\n")
		fid.write("#include <OB/Properties.h>\n")
		fid.write("#include \"FactoryAction%s_skel_orba.hh\"\n"%(factorydef.name))
		for servertype in factorydef.GetServertypeList():
			fid.write("#include \"%s_orba.hh\"\n"%(servertype))
		fid.write("\n")

		# Visibroker specific includes
		fid.write("#elif defined(VISIBROKER)\n")
		fid.write("#include \"FactoryAction%s_server_visi.hh\"\n"%(factorydef.name))
		for servertype in factorydef.GetServertypeList():
			fid.write("#include \"%s_client_visi.hh\"\n"%(servertype))
		fid.write("\n")

		# ORBIX specific includes
		fid.write("#elif defined(ORBIX)\n")
		fid.write("#include \"FactoryAction%s_server_orbi.hh\"\n"%(factorydef.name))
		for servertype in factorydef.GetServertypeList():
			fid.write("#include \"%s_client_orbi.hh\"\n"%(servertype))
		fid.write("\n")

		# Not implemented ORB
		fid.write("#else\n")
		fid.write("#error Unsupported ORB\n")
		fid.write("#endif\n")
		fid.write("\n")

  		# Standard includes
		fid.write("#include <unistd.h>\n")
		fid.write("#include <iostream.h>\n")
		fid.write("#include <fstream.h>\n")
		fid.write("#include <string.h>\n")
		fid.write("\n")

		# Automatic Activation includes
		fid.write("#include <OrbBind.h>\n")
		for servertype in factorydef.GetServertypeList():
			fid.write("DEF_OrbBind(%s)\n"%(servertype))
			
			
		# Implementation class description
		#----------------------------------
		fid.write("/*\n")
		fid.write("CLASS\n")
		fid.write("\tFactoryAction%s\n"%(factorydef.name))
		fid.write("\n")
		fid.write("DESCRIPTION\n")
		fid.write("\tClass derives from FactoryAction\n")
		fid.write("\n")
		fid.write("AUTHOR\n")
		fid.write("\tDassault Aviation\n")
		fid.write("\n")
		fid.write("*/\n")
		fid.write("\n")

		# Implementation class definition
		#----------------------------------
		fid.write("class FactoryAction%s%s:\n"%(factorydef.name,self._files['impl_suff']))
		fid.write(" public virtual SKELETON(FactoryAction%s),\n"%(factorydef.name))
		fid.write(" public virtual FactoryAction_impl\n")
		fid.write("{\n")
		
		# Public methods
		fid.write("\tpublic:\n")
		fid.write("\n")
		fid.write("\t\t// Constructor\n")
		fid.write("\t\t// @input obj_name Name given to the FactoryAction\n")
		fid.write("\t\tFactoryAction%s%s(const char * obj_name);\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("\t\t// Destructor\n")
		fid.write("\t\t~FactoryAction%s%s();\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("\t\t// Action creation\n")
		fid.write("\t\t// @input action_name Name of the Action Object to be created, must be one of those managed by the FactoryAction\n")
		fid.write("\t\t// @return Pointer on a new Action Object\n")
		fid.write("\t\tAction_ptr createAction(const char * action_name);\n")
		fid.write("\n")
		fid.write("\t\t// Delete the reference of an object\n")
		fid.write("\t\t// @input type_name Type name of the Factory which created the object\n")
		fid.write("\t\t// @input host_name Name of the host where the object has been created\n")
		fid.write("\t\t// @input obj_name Name of the object to delete\n")
		fid.write("\t\t// @return 0 on success\n")
		fid.write("\t\tCORBA(Short) deleteFactObj(const char * type_name, const char * host_name, const char * obj_name);\n")
		fid.write("\n")

		# Protected methods
		fid.write("\tprotected:\n")
		fid.write("\n")
		fid.write("\t\t// Binding a Factory\n")
		fid.write("\t\tCORBA(Object_ptr) bindFactory(const char * name, const char * machine_name,\n")
		fid.write("\t\t                              const char * fact_obj_name, CORBA(UShort) fact_port_number);\n")
		fid.write("\n")

		for actionname in factorydef.actions.keys():
			fid.write("\t\t// Creating the Action Object \"Action%s\"\n"%(actionname))
			fid.write("\t\t// @input name Name given to the Action Object\n")
			fid.write("\t\t// @return Pointer on a new Action%s Object\n"%(actionname))
			fid.write("\t\tAction%s%s createAction%s(const char * name);\n"%(actionname,self._files['ptr_suff'],actionname))
			fid.write("\n")

		# End
		#modifs pierre. old : fid.write("}\n")
		fid.write("};\n")
		fid.write("#endif\n")
		
		
		
	def _generateActionHeader(self,actiondef,fid):
		
		# Titles and Commentary
		#----------------------
		self._generateGenericHeader(fid)

		fid.write("/**********************************************************************\n")
		fid.write("\n")
		fid.write("Application Action Factory : Action%s%s.h\n"%(actiondef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("Description : Action%s header implementation\n"%(actiondef.name))
		fid.write("\n")
		fid.write("**********************************************************************/\n")
		fid.write("\n")
		fid.write("#ifndef Action%s%s_h\n"%(actiondef.name,self._files['impl_suff']))
		fid.write("#define Action%s%s_h\n"%(actiondef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("#include <string.h>\n")
		fid.write("#include <iostream.h>\n")
		fid.write("#include <strstream.h>\n")
		fid.write("#include <fstream.h>\n")
		fid.write("#include <stdlib.h>\n")
		fid.write("#include <stdio.h>\n")
		fid.write("\n")
		fid.write("#include \"Corba_def.h\"\n")
		fid.write("#include \"Action%s\"\n"%(self._files['impl_header_suff']))
		fid.write("\n")
		
		# Includes
		#---------
		# Specific ORBacus includes  
		fid.write("#if defined(ORBACUS)\n")
		fid.write("#include \"OB/CORBA.h\"\n")
		fid.write("#include \"OB/Util.h\"\n")
		fid.write("#include \"ModifyList.h\"\n")
		fid.write("#include \"Action%s_skel_orba.hh\"\n"%(actiondef.name))
		for objecttypename in actiondef.GetObjecttypenameList():
			fid.write("#include \"%s_orba.hh\"\n"%(objecttypename))
		for servertype in actiondef.GetServertypeList():
			fid.write("#include \"%s_orba.hh\"\n"%(servertype))
		fid.write("\n")
		
		# Specific Visibroker includes
		fid.write("#elif defined(VISIBROKER)\n")
		fid.write("#include \"Action%s_server_visi.hh\"\n"%(actiondef.name))
		for objecttypename in actiondef.GetObjecttypenameList():
			fid.write("#include \"%s_client_visi.hh\"\n"%(objecttypename))
		for servertype in actiondef.GetServertypeList():
			fid.write("#include \"%s_client_visi.hh\"\n"%(servertype))
		fid.write("\n")

		# Unknown ORB
		fid.write("#else\n")
		fid.write("#error Unsupported ORB\n")
		fid.write("#endif\n")
		fid.write("\n")

		# Class definition
		#-----------------
		fid.write("class Action%s%s:\n" %(actiondef.name,self._files['impl_suff']))
		fid.write(" public virtual SKELETON(Action%s),\n"%(actiondef.name))
		fid.write(" public virtual Action_impl\n")
		fid.write("{\n")
		fid.write("\n")

		# Public methods
		fid.write("\tpublic:\n")
		fid.write("\t\t// Constructor\n")
		fid.write("\t\tAction%s%s(\n"%(actiondef.name,self._files['impl_suff']))
		fid.write("#if defined(ORBACUS)\n")
		fid.write("\t\t\tCORBA(ORB_ptr) o,\n")
		fid.write("\t\t\tCORBA(BOA_ptr) b,\n")
		fid.write("#endif\n")
		fid.write("\t\t\tconst char *object_name = NULL);\n")
		fid.write("\n")
		fid.write("\t\t// Destructor\n")
		fid.write("\t\t~Action%s%s();\n"%(actiondef.name,self._files['impl_suff']))
		fid.write("\n") 
		fid.write("\t\t// Action execution\n")
		fid.write("\t\tCORBA(Short) exec();\n")
		fid.write(" };\n")
		fid.write("\n") 
		fid.write("#endif\n")
		
		
		
	def _generateMakefileSource(self,factorydef,fid):
		self._generateMakefileGenericHeader(fid)
		fid.write("include $(STD_RULES_DIR)/make.src.top\n")
		fid.write("\n")
		fid.write("#==================================================\n")
		fid.write("# Skeleton & Implementation objects\n")
		fid.write("#--------------------------------------------------\n")
		fid.write("\n")
		fid.write("OBJECTS_SKEL=\\\n")
		fid.write(" FactoryAction%s$(ORB_IMPL_SUFF)$(OBJ_SUFF)\\\n"%(factorydef.name))
		fid.write(" FactoryAction%s$(ORB_SKEL_SUFF)$(OBJ_SUFF)"%(factorydef.name))
		i=1
		for actionname in factorydef.actions.keys():
			fid.write(" \\\n Action%s$(ORB_IMPL_SUFF)$(OBJ_SUFF)\\\n"%(actionname))
			fid.write(" Action%s$(ORB_SKEL_SUFF)$(OBJ_SUFF)"%(actionname))
			i= i+1
		fid.write("\n")
		fid.write("\n")
		fid.write("#==================================================\n")
		fid.write("# Stub objects\n")
		fid.write("#--------------------------------------------------\n")
		fid.write("OBJECTS_STUB=\\\n")
		fid.write(" FactoryAction%s$(ORB_STUB_SUFF)$(OBJ_SUFF) "%(factorydef.name))
		i=1
		for actionname in factorydef.actions.keys():
			fid.write(" \\\n Action%s$(ORB_STUB_SUFF)$(OBJ_SUFF)"%(actionname))
			i= i+1
		fid.write("\n")
		fid.write("\n")
		fid.write("include $(STD_RULES_DIR)/make.srcbin.bottom\n")
		
		
		
	def _generateFactorySource(self,factorydef,fid):
	
		# Action description parameters, input & output
		#-----------------------------------------------
		nb_actions		= len(factorydef.actions)
		nb_servers		= len(factorydef.GetServertypeList())
		nb_objecttypes	= len(factorydef.GetObjecttypenameList())

		# Titles and Commentary
		#-----------------------
		self._generateGenericHeader(fid)

		fid.write("/**********************************************************************\n")
		fid.write("\n")
		fid.write("Application Action Factory : FactoryAction%s%s.C\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("Description : FactoryAction%s implementation\n"%(factorydef.name))
		fid.write("\n")
		fid.write("**********************************************************************/\n")
		fid.write("\n")
		fid.write("#include <FactoryAction%s%s.h>\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\n")

		# Constructor method
		#--------------------
		fid.write("FactoryAction%s%s::\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("FactoryAction%s%s(const char *object_name) \n"%(factorydef.name,self._files['impl_suff']))
		fid.write(" : FactoryAction_impl(object_name,%d,%d)\n"%(nb_actions,nb_servers))
		fid.write("{\n")
		fid.write("\tif (_traces>=DETAIL) cout << \"APPLICATION FactoryAction%s%s\" << endl;\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\tif (_traces>=DETAIL) cout << \"FactoryAction%s%s : Constructor\" << endl;\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\tif (_traces>=DETAIL) cout << endl;\n")
		fid.write("\tif (_traces>=DETAIL) cout << flush;\n")
		fid.write("\n")
		fid.write("\tif (_NS_name) delete [] _NS_name;\n")
		fid.write("\t_NS_name = strdup(\"FactoryAction%s\");\n"%(factorydef.name))
		fid.write("\n")

  		# Action list initialisation code
		fid.write("\t/******* Actions Sequences ********/\n")
		fid.write("\t// Initialisation and filling the list of available actions\n")
		fid.write("\t// NB : given order has to be the same as in ENUM\n")
		fid.write("\t// in file FactoryAction%s.%s\n"%(factorydef.name,self._files['idl_suff']))
		fid.write("\ttab_action.length(%d);\n"%(nb_actions))
		i=0
		for actionname in factorydef.actions.keys():
			fid.write("\ttab_action[%d] = CORBA(string_dup)(\"Action%s\");\n"%(i,actionname))
			i= i+1
		fid.write("\n")

		# Factory Objects list initialisation code
		fid.write("\t/******* Factory Objects Sequences ********/\n")
		fid.write("\t// Initialisation and filling the list of needed FactoryObjects\n")
		fid.write("\t// NB : given order has to be the same as in ENUM_FACT\n")
		fid.write("\t// in file FactoryAction%s.%s\n"%(factorydef.name,self._files['idl_suff']))
		fid.write("\ttab_fact.length(%d);\n"%(nb_servers))
		i=0
		for servertype in factorydef.GetServertypeList():
			fid.write("\ttab_fact[%d] = CORBA(string_dup)(\"%s\");\n"%(i,servertype))
			i= i+1
		fid.write("}\n")
		fid.write("\n")

		# Destructor Method
		#-------------------
		fid.write("FactoryAction%s%s::~FactoryAction%s%s()\n"%(factorydef.name,self._files['impl_suff'],factorydef.name,self._files['impl_suff']))
		fid.write("{\n")
		fid.write("\tif (_traces>=DETAIL ) cout << \"FactoryAction%s%s : Destructor\" << endl << endl;\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("}\n")
		fid.write("\n")

		# Create Action Method
		#----------------------
		fid.write("Action_ptr FactoryAction%s%s::createAction(const char * name)\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("{\n")
		fid.write("\tif (_traces>=DETAIL) cout << \"FactoryAction%s%s : Method Creating Action\" << endl;\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("\tAction_var new_Action = Action::_nil();\n")
		fid.write("\tCORBA(Short) nbA;\n")
		fid.write("\n")
		fid.write("\t// Looking for the number of action in the action list\n")
		fid.write("\tnbA = find_action(name);\n")
		fid.write("\n")
		fid.write("\tif (!(nbA == -1)) {\n")
		fid.write("\t\tswitch (ENUM_ACTION(nbA)) {\n")
		fid.write("\n")
		for actionname in factorydef.actions.keys():
			fid.write("\t\t\t// Creation of an action %s and cast to Action_ptr\n"%(actionname))
			fid.write("\t\t\tcase %s:\n"%(actionname))
			fid.write("\t\t\t\tnew_Action = createAction%s(name);\n"%(actionname))
			fid.write("\t\t\t\tbreak;\n")
			fid.write("\n")
		fid.write("\t\t\tdefault:\n")
		fid.write("\t\t\t\tcout << \"Non listed Action \" << endl;\n")
		fid.write("\t\t\t\tnew_Action = Action::_nil();\n")
		fid.write("\t\t\t\tbreak;\n")
		fid.write("\t\t};\n")
		fid.write("\t}\n")
		fid.write("\n")
		fid.write("\treturn Action::_duplicate(new_Action);\n")
		fid.write("}\n")
		fid.write("\n")

		# Create Action X Methods
		#-------------------------
		for actionname in factorydef.actions.keys():
			fid.write("Action%s%s FactoryAction%s%s::createAction%s(const char * name)\n"%(actionname,self._files['ptr_suff'],factorydef.name,self._files['impl_suff'],actionname))
			fid.write("{\n")
			fid.write("\tif (_traces>=DETAIL ) cout << \" FactoryAction%s%s : Method Creating Action%s\" << endl;\n"%(actionname,self._files['impl_suff'],actionname))
			fid.write("\n")
			fid.write("#if defined(VISIBROKER)\n")
			fid.write("\tCORBA::BOA_var boa = _boa();\n")
			fid.write("\tboa->scope(CORBA::BOA::SCOPE_LOCAL);\n")
			fid.write("\n")
			fid.write("\tAction%s%s *new_Action%sObj = new Action%s%s((const char *)name);\n"%(actionname,self._files['impl_suff'],actionname,actionname,self._files['impl_suff']))
			fid.write("\tobj_entry_var new_obj_ent = new obj_entry;\n")
			fid.write("\n")
			fid.write("\tnew_obj_ent->obj_name = (const char *)name;\n")
			fid.write("\tnew_obj_ent->obj_ref = ((CORBA::Object *) new_Action%sObj);\n"%(actionname))
			fid.write("\n")
			fid.write("\tAction%s::_duplicate(new_Action%sObj);\n"%(actionname,actionname))
			fid.write("\tobject_list.AppendEntry(*new_obj_ent);\n")
			fid.write("\n")
			fid.write("\tboa->obj_is_ready(new_Action%sObj);\n"%(actionname))
			fid.write("\treturn new_Action%sObj;\n"%(actionname))
			fid.write("\n")
			fid.write("#elif defined(ORBACUS)\n")
			fid.write("\tAction%s%s new_obj = Action%s::_nil();\n"%(actionname,self._files['var_suff'],actionname))
			fid.write("\tnew_obj = new Action%s%s(orb,boa,name);\n"%(actionname,self._files['impl_suff']))
			fid.write("\n")
			fid.write("\tobj_entry2 new_obj_ent;\n")
			fid.write("\tnew_obj_ent.obj_name = CORBA(string_dup)(name);\n")
			fid.write("\tnew_obj_ent.obj_ref = CORBA(Object)::_duplicate( new_obj);\n")
			fid.write("\tobject_list.AppendEntry(new_obj_ent);\n")
			fid.write("\n")
			fid.write("\treturn Action%s::_duplicate(new_obj);\n"%(actionname))
			fid.write("#else\n")
			fid.write("#error Unsupported ORB\n")
			fid.write("#endif\n")
			fid.write("}\n")
			fid.write("\n")
	
		# BindFactory Method
		#--------------------
		fid.write("CORBA(Object_ptr) FactoryAction%s%s::bindFactory(const char * name, const char * machine_name,\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\tconst char * fact_obj_name, CORBA(UShort) fact_port_number )\n")
		fid.write("{\n")
		fid.write("\tif (_traces>=DETAIL ) cout << \"FactoryAction%s%s : Method binding Factory\" << endl;\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("\tCORBA(Object_var) fact = CORBA(Object)::_nil();\n")
		fid.write("\n")
		fid.write("\t// Looking for a reference on a existing FactoryObjects in fact_list\n")
		fid.write("\tint counter = 0;\n")
		fid.write("\tobj_fact *fact1 = fact_list.FirstEntry();\n")
		fid.write("\n")
		fid.write("\twhile (fact1 != NULL) {\n")
		fid.write("\t\tif (strcmp(fact1->fact_type_name, name) == 0 && strcmp(fact1->fact_machine_name, machine_name) == 0\n")
		fid.write("\t\t&& strcmp(fact1->fact_object_name, fact_obj_name) == 0 && fact1->fact_port_number==fact_port_number ) {\n")
		fid.write("\t\t\tif (_traces>=DETAIL) cout << name << \" already bound\" << endl;\n")
		fid.write("\t\t\tfact =  CORBA(Object)::_duplicate(fact1->fact_ref );\n")
		fid.write("\t\t\treturn fact._retn();\n")
		fid.write("\t\t}\n")
		fid.write("\t\tcounter++;\n")
		fid.write("\t\tfact1 = fact_list.NextEntry();\n")
		fid.write("\t}\n")
		fid.write("\n")
		fid.write("\t// Looking for the action number in the list of the available action\n")
		fid.write("\tCORBA(Short) nbF;\n")
		fid.write("\tCORBA(Object_var) obj;\n")
		fid.write("\n")
		fid.write("\tnbF = find_fact(name);\n")
		fid.write("\n")
		fid.write("\tif (!(nbF == -1)) {\n")
		fid.write("\t\tswitch (ENUM_FACT(nbF)) {\n")
		i=0
		for servertype in factorydef.GetServertypeList():
			fid.write("\t\t\tcase SERVER_%s:\n"%(servertype))
			fid.write("\n")
			fid.write("\t\t\t\t// If no Factory Objects reference found in fact_list\n")
			fid.write("\t\t\t\t// bind of a Factory Objects\n")
			fid.write("\t\t\t\ttry\n")
			fid.write("\t\t\t\t{\n")
			fid.write("#if defined(ORBACUS)\n")
			fid.write("\t\t\t\t\t// activation automatique\n")
			fid.write("\t\t\t\t\tif (fact_port_number == 1) {\n")
			fid.write("\t\t\t\t\t\tchar* server = NULL;\n")
			fid.write("\t\t\t\t\t\tserver = strdup(fact_obj_name);\n")
			fid.write("\t\t\t\t\t\tchar* marker = NULL;\n")
			fid.write("\t\t\t\t\t\tmarker = strdup(fact_obj_name);\n")
			fid.write("\t\t\t\t\t\tchar * hostname = NULL;\n")
			fid.write("\t\t\t\t\t\thostname = strdup(machine_name);\n")
			fid.write("\t\t\t\t\t\tif (_traces>=INFO) cout << \"Bind Server on   : hostname = \" << hostname << endl;\n")
			fid.write("\t\t\t\t\t\tif (_traces>=INFO) cout << \"  referenced by  : server = \" << server << endl;\n")
			fid.write("\t\t\t\t\t\tif (_traces>=INFO) cout << \"         and by  : marker = \" << marker << endl;\n")
			fid.write("\t\t\t\t\t\tobj = OrbBind(%s)(hostname, marker, server);\n"%(servertype))
			fid.write("\t\t\t\t\t} else {\n")
			fid.write("\t\t\t\t\t\tobj = orb->get_inet_object(machine_name,(CORBA(UShort)) fact_port_number ,fact_obj_name);\n")
			fid.write("\t\t\t\t\t}\n")
			fid.write("#else\n")
			fid.write("#error Unsupported ORB\n")
			fid.write("#endif\n")
			fid.write("\t\t\t\t}\n")
			fid.write("\t\t\t\tcatch(CORBA(SystemException)& ex)\n")
			fid.write("\t\t\t\t{\n")
			fid.write("\t\t\t\t\tif (_traces>=INFO) cout << \"Error binding to Server : System exception\" << endl;\n")
			fid.write("\t\t\t\t\tOB(PrintException)(ex);\n")
			fid.write("\t\t\t\t\treturn CORBA(Object)::_nil();\n")
			fid.write("\t\t\t\t}\n")
			fid.write("\t\t\t\tcatch(const CORBA(Exception)& excep)\n")
			fid.write("\t\t\t\t{\n")
			fid.write("\t\t\t\t\tif (_traces>=INFO) cout << \"Error binding to %s : CORBA exception\" << endl;\n"%(servertype))
			fid.write("\t\t\t\t\treturn CORBA(Object)::_nil();\n")
			fid.write("\t\t\t\t}\n")
			fid.write("\n")

			fid.write("\t\t\t\ttry\n")
			fid.write("\t\t\t\t{\n")
			fid.write("\t\t\t\t\tfact = %s::_narrow(obj);\n"%(servertype))
			fid.write("\t\t\t\t}\n")
			fid.write("\t\t\t\tcatch(CORBA(SystemException)& ex)\n")
			fid.write("\t\t\t\t{\n")
			fid.write("\t\t\t\t\tif (_traces>=INFO) cout << \"Error narrowing to %s : System exception\" << endl;\n"%(servertype))
			fid.write("\t\t\t\t\tOB(PrintException)(ex);\n")
			fid.write("\t\t\t\t\treturn CORBA(Object)::_nil();\n")
			fid.write("\t\t\t\t}\n")
			fid.write("\t\t\t\tcatch(const CORBA(Exception)& excep)\n")
			fid.write("\t\t\t\t{\n")
			fid.write("\t\t\t\t\tif (_traces>=INFO) cout << \"Error narrowing to %s : CORBA exception\" << endl;\n"%(servertype))
			fid.write("\t\t\t\t\treturn CORBA(Object)::_nil();\n")
			fid.write("\t\t\t\t}\n")
			fid.write("\n")

			fid.write("\t\t\t\tif (_traces>=DETAIL) cout << \"Creating a new %s\" << endl;\n"%(servertype))
			fid.write("\t\t\t\ttry\n")
			fid.write("\t\t\t\t{\n")
			fid.write("\t\t\t\t\tobj_fact new_obj_fact;\n")
			fid.write("\t\t\t\t\t// Add reference in fact_list\n")
			fid.write("\t\t\t\t\t// Fill a  obj_fact structure\n")
			fid.write("\t\t\t\t\tnew_obj_fact.fact_type_name = CORBA(string_dup)(name);\n")
			fid.write("\t\t\t\t\tnew_obj_fact.fact_machine_name = CORBA(string_dup)(machine_name);\n")
			fid.write("\t\t\t\t\tnew_obj_fact.fact_object_name = CORBA(string_dup)(fact_obj_name);\n")
			fid.write("\t\t\t\t\tnew_obj_fact.fact_port_number = CORBA(UShort)(fact_port_number);\n")
			fid.write("\t\t\t\t\tnew_obj_fact.fact_ref = CORBA(Object)::_duplicate(fact);\n")
			fid.write("\n")
			fid.write("\t\t\t\t\t// Insert the structure obj_fact in fact_list\n")
			fid.write("\t\t\t\t\tfact_list.AppendEntry(new_obj_fact);\n")
	
			fid.write("\t\t\t\t}\n")
			fid.write("\t\t\t\tcatch(const CORBA(SystemException)& excep) \n")
			fid.write("\t\t\t\t{\n")
			fid.write("\t\t\t\t\tif (_traces>=INFO) cout << \"Error Appending New Factory \" << fact_obj_name << \" in ActionFactory\"  << endl;\n")
			fid.write("\t\t\t\t\tOB(PrintException)(excep);\n")
			fid.write("\t\t\t\t\treturn CORBA(Object)::_nil();\n")
			fid.write("\t\t\t\t}\n")
			fid.write("\n")
			fid.write("\t\t\t\tbreak;\n")
			fid.write("\n")

		fid.write("\t\t\tdefault:\n")
		fid.write("\t\t\t\tif (_traces>=INFO) cout << \"ERROR : Non Listed Factory\" << endl;\n")
		fid.write("\t\t\t\tbreak;\n")
		fid.write("\t\t};\n")
		fid.write("\t}\n")

		fid.write("\n")
		fid.write("\treturn fact._retn();\n")
		fid.write("}\n")
		fid.write("\n")

		# Delete FactObj Method
		#--------------------------
		fid.write("CORBA(Short) FactoryAction%s%s::deleteFactObj(const char * type_name,\n"%(factorydef.name,self._files['impl_suff']))
		fid.write(" const char * machine_name, const char * obj_name)\n")
		fid.write("{\n")
		fid.write("\tif (_traces>=DETAIL) cout << \"FactoryAction%s%s : Method deleting Factory Object\" << endl;\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\t// static char * ProcId = \"@(#) FactoryAction%s%s::DeleteFactObj $Id:$\";\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("\t// Delete object obj_name created in factory of type type_name on host machine_name\n")
		fid.write("\n")
		fid.write("\tint counter = 0;\n")
		fid.write("\tCORBA(Short) nbF;\n")
		fid.write("\tobj_fact *fact1 = fact_list.FirstEntry();\n")
		fid.write("\n")
		fid.write("\twhile (fact1 != NULL) {\n")
		fid.write("\t\tif (strcmp(fact1->fact_type_name, type_name) == 0 && strcmp(fact1->fact_machine_name, machine_name) == 0 ) {\n")
		fid.write("\t\t\t// Looking for the number of action in the action_list\n")
		fid.write("\t\t\tnbF = find_fact(type_name);\n")
		for servertype in factorydef.GetServertypeList():
			fid.write("\t\t\t\t%s%s %sObj;\n"%(servertype,self._files['var_suff'],servertype))
		fid.write("\t\t\t\tif (!(nbF == -1)) {\n")
		fid.write("\t\t\t\t\tswitch (ENUM_FACT(nbF)) {\n")
		for servertype in factorydef.GetServertypeList():
			fid.write("\t\t\t\t\t\tcase SERVER_%s:\n"%(servertype))
			fid.write("\t\t\t\t\t\t\t%sObj = %s::_narrow(fact1->fact_ref);\n"%(servertype,servertype))
			fid.write("\t\t\t\t\t\t\t%sObj->deleteObject(obj_name);\n"%(servertype))
			fid.write("\t\t\t\t\t\t\tbreak;\n")
		fid.write("\t\t\t\t\t}\n")
		fid.write("\t\t\t\t} else {\n")
		fid.write("\t\t\t\t\tif (_traces>=INFO) cout << \"Object from a factory Deletion failed: Factory not listed\" << endl;\n")
		fid.write("\t\t\t\t}\n")
		fid.write("\t\t\t\treturn TRUE;\n")
		fid.write("\t\t\t}\n")
		fid.write("\t\t\tcounter++;\n")
		fid.write("\t\t\tfact1 = fact_list.NextEntry();\n")
		fid.write("\t}\n")
		fid.write("\n")
		fid.write("\tif (_traces>=INFO) cout << \"Deletion failed: Factory not found\" << endl;\n")
		fid.write("\treturn FALSE;\n")
		fid.write("}\n")
	
	
	
	def _generateActionSource(self,actiondef,fid):
		
		# Action description parameters, input & output
		#----------------------------------------------
		nb_input		= len(actiondef.inputPorts)
		nb_output		= len(actiondef.outputPorts)
		nb_parameter	= len(actiondef.parameters)
		nb_factory		= len(actiondef.GetServertypeList())
		nb_objecttypes	= len(actiondef.GetObjecttypenameList())

		# Titles and Commentary
		#----------------------
		self._generateGenericHeader(fid)

		fid.write("/**********************************************************************\n")
		fid.write("\n")
		fid.write("Application Action Factory : Action%s%s.C\n"%(actiondef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("Description : Action%s implementation\n"%(actiondef.name))
		fid.write("\n")
		fid.write("**********************************************************************/\n")
		fid.write("\n")
		fid.write("#include <Action%s%s.h>\n"%(actiondef.name,self._files['impl_suff']))
		fid.write("\n")

		# Constructor
		#------------
		fid.write("// Constructor\n")
		fid.write("Action%s%s::Action%s%s(\n"%(actiondef.name,self._files['impl_suff'],actiondef.name,self._files['impl_suff']))
		fid.write("#if defined(ORBACUS)\n")
		fid.write("\tCORBA(ORB_ptr) o,\n")
		fid.write("\tCORBA(BOA_ptr) b,\n")
		fid.write("#endif\n")
		fid.write("const char *object_name) :\n")
		fid.write("\tAction%s(\n"%(self._files['impl_suff']))
		fid.write("#if defined(ORBACUS)\n")
		fid.write("\t\to,b,\n")
		fid.write("#endif\n")
		fid.write("object_name)\n")
		fid.write("{\n")
		fid.write("\tif (_traces >= DETAIL) cout << \"APPLICATION Action%s%s\" << endl;\n"%(actiondef.name,self._files['impl_suff']))
		fid.write("\tif (_traces >= DETAIL) cout << \"Action%s%s : Constructor \" << object_name << endl;\n"%(actiondef.name,self._files['impl_suff']))
		fid.write("\tif (_traces >= DETAIL) cout << endl;\n")
		fid.write("\n")

		# Objects Types sequences initialisation
		fid.write("	/******* Objects Types Sequence ********/\n")
		fid.write("	Types_seq.length(%d);\n"%(nb_objecttypes))
		fid.write("\n")
  		i=0
		for objecttype in actiondef.GetObjecttypenameList():
			fid.write("\tTypes_seq[%d] = CORBA(string_dup)(\"%s\");\n"%(i,objecttype))
			i= i+1
		
		# Inputs initialisation code
		fid.write("\n")
		fid.write("	/******* Input Sequences ********/\n")
		fid.write("	InputTypes_seq.length(%d);\n"%(nb_input))
		fid.write("	InputObjBdd_seq.length(%d);\n"%(nb_input))
		fid.write("	InputNames_seq.length(%d);\n"%(nb_input))
		fid.write("	InputDesc_seq.length(%d);\n"%(nb_input))
		fid.write("\n")
		i=0
		for inputportname in actiondef.inputorder:
			fid.write("\t// Initialisation of input %d\n"%(i))
			fid.write("\tInputTypes_seq[%d] = CORBA(string_dup)(\"%s\");\n"%(i,actiondef.inputPorts[inputportname].type))
			#modif any                  fid.write("\tInputObjBdd_seq[%d] = CORBA(Object)::_nil();\n"%(i))
			fid.write("\tInputNames_seq[%d] = CORBA(string_dup)(\"%s\");\n"%(i,actiondef.inputPorts[inputportname].name))
			fid.write("\tInputDesc_seq[%d]  = CORBA(string_dup)(\"%s\");\n"%(i,actiondef.inputPorts[inputportname].desc))
			fid.write("\n")
			i= i+1

		# Parameters initialisation code
		fid.write("\t/******* Parameters Sequences ********/\n")
		fid.write("\tParameterTypes_seq.length(%d);\n"%(nb_parameter))
		fid.write("\tParameterNames_seq.length(%d);\n"%(nb_parameter))
		fid.write("\tParameterDesc_seq.length(%d);\n"%(nb_parameter))
		fid.write("\tParameterValueDesc_seq.length(%d);\n"%(nb_parameter))
		fid.write("\n")
		i=0
		for paramname in actiondef.paramorder:
			fid.write("\t// Initialisation of parameter %d (%s)\n"%(i,actiondef.parameters[paramname].name))
			fid.write("\tParameterTypes_seq[%d]     = CORBA(string_dup)(\"%s\");\n"%(i,actiondef.parameters[paramname].type))
			fid.write("\tParameterNames_seq[%d]     = CORBA(string_dup)(\"%s\");\n"%(i,actiondef.parameters[paramname].name))
			fid.write("\tParameterDesc_seq[%d]      = CORBA(string_dup)(\"%s\");\n"%(i,actiondef.parameters[paramname].desc))
			fid.write("\tParameterValueDesc_seq[%d] = CORBA(string_dup)(\"\");\n"%(i))
			fid.write("\n")
			i= i+1

		# Outputs initialisation code
		fid.write("\t/******* Outputs Sequences ********/\n")
		fid.write("\tOutputTypes_seq.length(%d);\n"%(nb_output))
		fid.write("\tOutputObjBdd_seq.length(%d);\n"%(nb_output))
		fid.write("\tOutputNames_seq.length(%d);\n"%(nb_output))
		fid.write("\tOutputDesc_seq.length(%d);\n"%(nb_output))
		fid.write("\tOutputValueDesc_seq.length(%d);\n"%(nb_output))
		fid.write("\n")
		i=0
		for outputportname in actiondef.outputorder:
			fid.write("\t// Initialisation of output %d\n"%(i))
			fid.write("\tOutputTypes_seq[%d]     = CORBA(string_dup)(\"%s\");\n"%(i,actiondef.outputPorts[outputportname].type))
			#modifs any                  fid.write("\tOutputObjBdd_seq[%d] = CORBA(Object)::_nil();"
			fid.write("\tOutputNames_seq[%d]     = CORBA(string_dup)(\"%s\");\n"%(i,actiondef.outputPorts[outputportname].name))
			fid.write("\tOutputDesc_seq[%d]      = CORBA(string_dup)(\"%s\");\n"%(i,actiondef.outputPorts[outputportname].desc))
			fid.write("\tOutputValueDesc_seq[%d] = CORBA(string_dup)(\"pointer\");\n"%(i))
			fid.write("\n")
			i= i+1

		# Factory initialisation code
		fid.write("\t/******* Factory Sequences ********/\n")
		fid.write("\tFactObj_list.length(%d);\n"%(nb_factory))
		fid.write("\tFactName_list.length(%d);\n"%(nb_factory))
		fid.write("\n")
		i=0
		for servertype in actiondef.GetServertypeList():
			fid.write("\t// Initialisation of Factory %d\n"%(i))
			fid.write("\tFactName_list[%d] = CORBA(string_dup)(\"%s\");\n"%(i,actiondef.servers[servertype].type))
			fid.write("\tFactObj_list[%d] = CORBA(Object)::_nil();\n"%(i))
			fid.write("\n")
			i= i+1

		fid.write("}\n")
		fid.write("\n")

		# Destructor
		#-----------
		fid.write("// Destructor\n")
		fid.write("Action%s%s::~Action%s%s()\n"%(actiondef.name,self._files['impl_suff'],actiondef.name,self._files['impl_suff']))
		fid.write("{\n")
		fid.write("\tif (_traces >= DETAIL) cout << \"Action%s%s : Destructor\" << endl << endl;\n"%(actiondef.name,self._files['impl_suff']))
		fid.write("}\n")
		fid.write("\n")

		# Exec method
		#-------------
		fid.write("// Exec method\n")
		fid.write("CORBA(Short) Action%s%s::exec()\n"%(actiondef.name,self._files['impl_suff']))
		fid.write("{\n")
		fid.write("\tif (_traces >= DETAIL) cout << \"Action%s%s : exec\" << endl << endl;\n"%(actiondef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("\t// Initialization\n")
		fid.write("\tCORBA(Short) ires = TRUE;\n")

  		# Try to get a reference for each used FactoryObject
		fid.write("\n")
		fid.write("\t/**************************************************\n")
		fid.write("\t\tGETTING REFERENCE ON FACTORY OBJECTS\n")
		fid.write("\t***************************************************/\n")
		fid.write("\n")
		i=0
		for servertype in actiondef.GetServertypeList():
			type= actiondef.servers[servertype].type
			fid.write("\t%s%s fact%d = %s::_nil();\n"%(type,self._files['var_suff'],i,type))
			fid.write("\ttry {\n")
			fid.write("\t\t\tfact%d = %s::_narrow(FactObj_list[%d]);\n"%(i,type,i))
			fid.write("\t}\n")
			fid.write("\tcatch(CORBA(SystemException)& ex) {\n")
			fid.write("\t\tif (_traces >= INFO) cout << \"System Error while narrowing %s in Action %s : \" << endl;\n"%(type,actiondef.name))
			fid.write("\t\tOB(PrintException)(ex);\n")
			fid.write("\t\treturn -1;\n")
			fid.write("\t}\n")
			fid.write("\t// Test on reference\n")
			fid.write("\tif (CORBA(is_nil)(fact%d)) {\n"%(i))
			fid.write("\t\tif (_traces >= INFO) cout << \"Error binding to %s\" << endl;\n"%(type))
			fid.write("\t\treturn -1;\n")
			fid.write("\t}\n")
			fid.write("\n")
			i= i+1
		fid.write("\n")

		# Try to get a reference for each used input object
		#--------------------------------------------------
		# Initialization of inputs
		fid.write("\n")
		fid.write("\t/****************************************\n")
		fid.write("\t\tGETTING REFERENCE ON INPUT OBJECTS\n")
		fid.write("\t****************************************/\n")
		fid.write("\n")
		
		fid.write("\tCORBA(Object%s) obj;\n"%(self._files['var_suff']))
		i=0
		for inputportname in actiondef.inputorder:
			name	= actiondef.inputPorts[inputportname].name
			type	= actiondef.inputPorts[inputportname].type
			desc	= actiondef.inputPorts[inputportname].desc
			category= actiondef.inputPorts[inputportname].category
			if category == "object":
				fid.write("\t// Object class: %s, number: %d\n"%(type,i))
				fid.write("\t%s%s input%d%s = %s::_nil();\n"%(type,self._files['var_suff'],i,type,type))
				fid.write("\ttry {\n")
				fid.write("\t\tInputObjBdd_seq[%d] >>= CORBA(Any::to_object)(obj);\n"%(i))
				fid.write("\t\tinput%d%s = %s::_narrow(obj);\n"%(i,type,type))
				fid.write("\t}\n")
				fid.write("\tcatch(CORBA(SystemException)& ex) {\n")
				fid.write("\t\tif (_traces >= INFO) cout << \"System Error while narrowing input%d%s in Action %s : \" << endl;\n"%(i,type,actiondef.name))
				fid.write("\t\tOB(PrintException)(ex);\n")
				fid.write("\t\treturn -1;\n")
				fid.write("\t}\n")
				fid.write("\t// Test on reference\n")
				fid.write("\tif (CORBA(is_nil)(input%d%s)) {\n"%(i,type))
				fid.write("\t\tif (_traces >= INFO) cout << \"Invalid input object input%d%s in Action %s : \" << endl;\n"%(i,type,actiondef.name))
				fid.write("\t\treturn -1;\n")
				fid.write("\t}\n")
			elif category == "sequence":
				fid.write("\t%s * input%d%s;\n"%(type,i,type))
				fid.write("\ttry {\n")
				fid.write("\t\tInputObjBdd_seq[%d] >>= input%d%s;\n"%(i,i,type))
				fid.write("\t}\n")
				fid.write("\tcatch(CORBA(SystemException)& ex) {\n")
				fid.write("\t\tif (_traces >= INFO) cout << \"System Error while narrowing input%d%s in Action %s : \" << endl;\n"%(i,type,actiondef.name))
				fid.write("\t\tOB(PrintException)(ex);\n")
				fid.write("\t\treturn -1;\n")
				fid.write("\t}\n")
				fid.write("\t// Test on reference\n")
				fid.write("\tif (input%d%s->length()==0) {\n"%(i,type))
				fid.write("\t\tif (_traces >= INFO) cout << \"Invalid input object input%d%s in Action %s : \" << endl;\n"%(i,type,actiondef.name))
				fid.write("\t\treturn -1;\n")
				fid.write("\t}\n")
			i= i+1
		fid.write("\n")

		# Initialization of output
		fid.write("\n")
		fid.write("\t/**********************************\n")
		fid.write("\t\tINITIALIZING OUTPUT\n")
		fid.write("\t**********************************/\n")
		fid.write("\n")
		
		i=0
		for outputportname in actiondef.outputorder:
			name	= actiondef.outputPorts[outputportname].name
			type	= actiondef.outputPorts[outputportname].type
			desc	= actiondef.outputPorts[outputportname].desc
			category= actiondef.outputPorts[outputportname].category
			if category == "object":
				fid.write("	%s%s output%d%s = %s::_nil();\n"%(type,self._files['var_suff'],i,type,type))
			elif category == "sequence":
				fid.write("	%s * output%d%s;\n"%(type,i,type))
			i= i+1
		fid.write("\n")
		
		# Getting parameters
		fid.write("\n")
		fid.write("\t/********************************************************** \n")
		fid.write("\t\tGETTING PARAMETERS\n")
		fid.write("\t\t\tAll Parameters are declared as String\n")
		fid.write("\t\t\tThis is developper responsability to cast parameters\n")
		fid.write("\t\t\tto the right type\n")
		fid.write("\t**********************************************************/\n")
		fid.write("\n")
		
		i=0
		for paramname in actiondef.paramorder:
			name= actiondef.parameters[paramname].name
			type= actiondef.parameters[paramname].type
			fid.write("\tCORBA(String_var) parameter%d%s = CORBA(string_dup)(ParameterValueDesc_seq[%d]);\n"%(i,type,i))
			i= i+1
		fid.write("\n")

		# User implementation file
		#-------------------------
		fid.write("\n")
		fid.write("\t/*****************************************************\n")
		fid.write("\t\tPART OF THE FILE THE DEVELOPER HAS TO COMPLETE\n")
		fid.write("\n")

		fid.write("\t\tInput Objects are already allocated and filled\n")
		i=0
		for inputportname in actiondef.inputorder:
			name	= actiondef.inputPorts[inputportname].name
			type	= actiondef.inputPorts[inputportname].type
			desc	= actiondef.inputPorts[inputportname].desc
			category= actiondef.inputPorts[inputportname].category
			fid.write("\t\t\tInput %d : input%d%s (%s:%s)\n"%(i,i,type,name,desc))
			i= i+1
		fid.write("\n")
		
		fid.write("\t\tParameters (string) are already allocated and filled\n")
		i=0
		for paramname in actiondef.paramorder:
			name= actiondef.parameters[paramname].name
			type= actiondef.parameters[paramname].type
			desc= actiondef.parameters[paramname].desc
			fid.write("\t\t\tParameter %d : parameter%d%s (%s:%s)\n"%(i,i,type,name,desc))
			i= i+1
		i=0
		fid.write("\n")

		fid.write("\t\tOutput Objects are already declarated but NOT created\n")
		i=0
		for outputportname in actiondef.outputorder:
			name	= actiondef.outputPorts[outputportname].name
			type	= actiondef.outputPorts[outputportname].type
			desc	= actiondef.outputPorts[outputportname].desc
			category= actiondef.outputPorts[outputportname].category
			fid.write("\t\t\tOutput %d : output%d%s (%s:%s)\n"%(i,i,type,name,desc))
			i= i+1
		fid.write("\n")
			
		fid.write("\t\tThe server (Factory Objects) you asked for are bounded\n")
		i=0
		for servertype in actiondef.GetServertypeList():
			type= actiondef.servers[servertype].type
			fid.write("\t\t\tFactory %d : fact%d (%s)\n"%(i,i,type))
			i= i+1
		fid.write("\n")

		fid.write("\t\tReturn of the method is 'ires' (TRUE by default)\n")
		fid.write("\t\t\t'ires' is a CORBA_Short\n")
		fid.write("\n")
		fid.write("\t\tBE CAREFUL: if an input object is also an output object \n")
		fid.write("\t\tThis is developer responsability to connect declared object\n")
		fid.write("\t\tor to change the output declaration and -Setting output- part \n")
		fid.write("\t*******************************************************/\n")
		fid.write("\n")
		fid.write("\t// Placing your code in an include will shield it\n")
		fid.write("\t// from beeing scratched by further automatic generation\n")
		fid.write("#include \"Exec%s%s.include\" \n"%(actiondef.name,self._files['impl_suff']))
		fid.write("\n")
		
		fid.write("\n")
		fid.write("\t/*****************************************************\n")
		fid.write("\t\tEND OF THE PART THE DEVELOPER HAS TO COMPLETE\n")
		fid.write("\t*******************************************************/\n")
		fid.write("\n")


		# Setting output
		fid.write("\n")
		fid.write("\t/**********************************\n")
		fid.write("\t\tSETTING OUTPUT\n")
		fid.write("\t**********************************/\n")
		fid.write("\n")
		
		i=0
		for outputportname in actiondef.outputorder:
			name	= actiondef.outputPorts[outputportname].name
			type	= actiondef.outputPorts[outputportname].type
			desc	= actiondef.outputPorts[outputportname].desc
			category= actiondef.outputPorts[outputportname].category
			fid.write("	OutputObjBdd_seq[%d] <<= output%d%s;\n"%(i,i,type))
			i= i+1
		fid.write("\n")
		
		i=0
		for paramname in actiondef.paramorder:
			name= actiondef.parameters[paramname].name
			type= actiondef.parameters[paramname].type
			fid.write("\tParameterValueDesc_seq[%d] = CORBA(string_dup)(parameter%d%s);\n"%(i,i,type))
			i= i+1
		fid.write("\n")

		# Cleaning
		#-------------------------
		fid.write("\n")
		fid.write("\t/**********************************\n")
		fid.write("\t// CLEANING\n")
		fid.write("\t**********************************/\n")
		fid.write("#ifdef VISIBROKER\n")
		fid.write("\tCORBA(release)(fact);\n")
		fid.write("\tCORBA(release)(FactObj_list[0]);\n")
		fid.write("#endif\n")
		fid.write("\n")
		fid.write("\treturn ires;\n")
		fid.write("}\n")
			
		
		
	def _generateFactoryServer(self,factorydef,fid):
	
		# Titles and Commentary
		#----------------------
		self._generateGenericHeader(fid)
		fid.write("\n")
		fid.write("#include \"FactoryAction%s%s.h\"\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("FactoryAction%s%s p;\n"%(factorydef.name,self._files['var_suff']))
		fid.write("\n")
		fid.write("void bye(int number) {\n")
		fid.write("\n")
		fid.write("	cout << \"Signal Caught : Exit on process :\" << number << endl;\n")
		fid.write("	cout << \"========== Destroy Factory =========\" << endl << endl;\n")
		fid.write("	if (p) p->destroy();\n")
		fid.write("	return ;\n")
		fid.write("}\n")
		fid.write("\n")
		
		fid.write("int main(int argc, char *argv[])\n")
		fid.write("{\n")
		fid.write("    signal(SIGINT,   bye);\n")
		fid.write("    signal(SIGQUIT,  bye);\n")
		fid.write("    signal(SIGILL,   bye);\n")
		fid.write("    signal(SIGFPE,   bye);\n")
		fid.write("    signal(SIGTERM,  bye);\n")
		fid.write("    signal(SIGIO,    bye);\n")
		fid.write("    signal(SIGSTOP,  bye);\n")
		fid.write("    signal(SIGTSTP,  bye);\n")
		fid.write("\n")
		
		fid.write("	if (argc != 3) {\n")
		fid.write("		cout << endl;\n")
		fid.write("		cout << \"usage : FactoryAction%s_server_ORBACUS <name> <portnumber>\" << endl;\n"%(factorydef.name))
		fid.write("		cout << \"name is the name of the FactoryAction you want to launch\" << endl;\n")
		fid.write("		cout << \"port number is the port number of the FactoryAction you want to launch\" << endl;\n")
		fid.write("		cout << endl;\n")
		fid.write("		return 1;\n")
		fid.write("	}\n")
		fid.write("\n")
		
		fid.write("	CORBA_ORB_var orb;\n")
		fid.write("	CORBA_BOA_var boa;\n")
		fid.write("    char * PC_ContextName;\n")
		fid.write("\n")
		
		fid.write("    try\n")
		fid.write("    {\n")
		fid.write("		// Initialize ORB and Basic Object Adaptor (BOA)\n")
		fid.write("		//orb = CORBA_ORB_init(argc, argv);\n")
		fid.write("		orb = OrbBind_ORB_init(argc, argv);\n")
		fid.write("		OBProperties * properties = OBProperties::instance();\n")
		fid.write("		properties->setProperty(\"ooc.boa.port\",argv[2]);\n")
		fid.write("		boa = orb->BOA_init(argc, argv);\n")
		fid.write("   	}\n")
		
		fid.write("	catch(CORBA_SystemException& ex)\n")
		fid.write("	{\n")
		fid.write("		OB(PrintException)(ex);\n")
		fid.write("		return (1);\n")
		fid.write("	}\n")
		fid.write("\n")
		
		fid.write("\n")
		fid.write("    try\n")
		fid.write("    {\n")
		fid.write("        PC_ContextName = new char[7];\n")
		fid.write("        strcpy(PC_ContextName,\"/jobs/\");\n")
		fid.write("\n")
		fid.write("        cout << \"========== Creation of FactoryAction%s=========\" << endl << endl;\n"%(factorydef.name))
		fid.write("		p = new FactoryAction%s%s(argv[1]);\n"%(factorydef.name,self._files['impl_suff']))
		fid.write("\n")
		fid.write("		orb->connect(p, argv[1]);\n")
		fid.write("\n")
		fid.write("		CORBA(String_var) ior = orb->object_to_string(p);\n")
		fid.write("		cout << ior << endl;\n")
		fid.write("\n")
		fid.write("#ifdef USE_NAMESERV\n")
		fid.write("        cout << \"========== Bind FactoryAction%s in Naming Context=========\" << endl << endl;\n"%(factorydef.name))
		fid.write("        p->bindInNamingContext(PC_ContextName);\n")
		fid.write("#endif\n")
		fid.write("\n")
		fid.write("        cout << \"========== Waiting for request=========\" << endl << endl;\n")
		fid.write("		boa->impl_is_ready(CORBA_ImplementationDef::_nil());\n")
		fid.write("\n")
		fid.write("        cout << \"========== Unbind FactoryAction%s from the Naming Service=========\" << endl << endl;\n"%(factorydef.name))
		fid.write("        p->unbindFromNamingContext();\n")
		fid.write("   	}\n")
		fid.write("	catch(CORBA_SystemException& ex)\n")
		fid.write("	{\n")
		fid.write("		OB(PrintException)(ex);\n")
		fid.write("		return (1);\n")
		fid.write("	}\n")
		fid.write("\n")
		fid.write("	return(0);\n")
		fid.write("}\n")
		fid.write("\n")

		
		
	def _createDir(self,dirname):
		"""Create directory"""
		if os.path.isdir(dirname):
			print "\t",dirname," already exist..."
		else:
			print "\t creating ",dirname
			try:
				os.mkdir(dirname)
			except:
				print "\n\n==> failed to create directory ",dirname
				sys.exit(1)
		
		
		
	def _checkbackup(self,filename):
		if self.backup and os.path.exists(filename):
			try:
				shutil.copyfile(filename,filename + ".bak")	
			except:
				print "\n\n==>failed to backup ",filename
				sys.exit(1)
			

def Generate(factorydef,dirname):
	ag=ActionGeneration(factorydef,dirname)
	print "================================================================="
	print "Automatic Action Factory generation v",ag._version
	print "(C) 2000 Dassault Aviation"
	print "-----------------------------------------------------------------"
	print "Generating C++ factory ",factorydef.name,"\nin ",dirname
	print "-----------------------------------------------------------------"
	ag.CreateDirs()
	ag.GenerateActions()
	ag.GenerateFactory()
	ag.GenerateMakefiles()
	print "End of generation"
	print "================================================================="
