#=========================================================================================
#@(#)Thierry: $Id$
#
#    Execute a single action
#
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Thierry CHEVALIER <t_chevalier@libertysurf.fr>
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 


from time import *
from threading import *

import sys
import string

from Fnorb.orb import CORBA,IOP

from lib.PortDef import *
from lib.ParameterDef import *
from lib.ActionDef import *
from lib.FactoryDef import *

import _GlobalIDL

frame = 0

#==================================================
# Global CORBA initialization
#--------------------------------------------------
print 'Initialising the ORB...'

# Initialise the ORB.
orb = CORBA.ORB_init(sys.argv, CORBA.ORB_ID)

#==================================================
# Action class
#--------------------------------------------------
class Action(Thread):

	def __init__(self,stringified_ior,factorydef,actiondef,actionbox):
		global frame
		global orb
		frame= frame + 1
		self.frame= frame

		Thread.__init__(self)
	
		self.stringified_ior= stringified_ior
		self.factorydef= factorydef
		self.actiondef= actiondef
		self.actionbox= actionbox

		print "Factory stringified IOR=",stringified_ior

		# Convert the stringified IOR into an active object reference
		# We can't use direct 'factory =orb.string_to_object(stringified_ior)',
		# because we need to cast object in a FactoryActionstringified_ior
		# We use the following explicit sequence instead
		# Convert the stringified IOR into an IOR instance.
		ior= IOP.IOR()
		ior._fnorb_from_string(stringified_ior)
		# Create an object reference from the IOR, with appropriate casting to FactoryAction
		self.factory = apply(_GlobalIDL.FactoryAction, (ior,))

		# Check we got a server
		if self.factory is None:
			raise 'Nil object reference for factory !'

		# Check we got the right server
		#URL='IDL:FactoryAction:1.0'
		#if not self.factory._is_a(URL):
		#	raise 'This is not an action factory server !'

		print "Factory ", self.factorydef.name, " connected for serving action ", self.actiondef.name, " !"
		print self.factory._FNORB_ID

	def SetInput(self,valuesin):
		self.valuesin= valuesin

	def SetParameters(self,parameters):
		self.parameters= parameters

	def SetServers(self, serversname, servershost, serversport):
		self.serversname= serversname
		self.servershost= servershost
		self.serversport= serversport


	def run(self):

		self.actionbox.Working()

		# Recover and package execution parameters
		actionname= "Action" + self.actiondef.name
		serverhost= string.strip(self.servershost.values()[0])
		servername= string.strip(self.serversname.values()[0])
		serverport= string.atoi(self.serversport.values()[0])
		print ">>> ",actionname," executed by ",servername," on ",serverhost," (port ",serverport,")"

		inputs_seq=[]
		print "    inputs:"
		for name in self.actiondef.inputorder:
			inputs_seq.append(self.valuesin[name])
			print "            ",name,"=",self.valuesin[name]

		parameters_in=[]
		print "    parameters in:"
		for name in self.actiondef.paramorder:
			parameters_in.append(self.parameters[name])
			print "            ",name,"=",self.parameters[name]

		outputs_seq=[]

		# Request factory server to execute action
		try:
			(outputs_seq,parameters_out)= self.factory.doit(actionname, inputs_seq, parameters_in, serverhost, servername, serverport)
		except:
			print "Error while executing ",actionname

		print "====================================="
		print "outputs_seq=",outputs_seq
		print "parameters=",parameters_out
		print "====================================="

		# It is more efficient for the action to directly post the output
		# values & parameters, but it cause this thread to require
		# execution of some Tk code where Tk is *not* thread safe.
		# This then cause unpredictable crashes (Tcl bytecode error...)
		
		# Post parameters back to actionbox
		i=0
		print "    parameters out:"
		for paramname in self.actiondef.paramorder:
			try:
				s="            %s= %s"%(paramname,parameters_out[i])
				print s
				self.actionbox.parameters[paramname].set_value(parameters_out[i])
			except:
				s="Unable to set parameter %s"%(paramname)
				print s
			i= i+1
		
		# Force all global shared parameters to update before firing ports
		self.actionbox.mapeditor.top.update_idletasks()

		# Post outputs back to actionbox
		i=0
		print "    outputs:"
		for portname in self.actiondef.outputorder:
			try:
				s="            %s (sortie %d)= %d"%(portname,i,outputs_seq[i])
				print s
				self.actionbox.SetOutputPort(portname,outputs_seq[i])
			except:
				s="Unable to set output port %s (i=%d)"%(portname,i)
				print s
			i= i+1
			
		# A revoir <!!!>
		if self.actionbox.voutputtrigger.get() == "on":
			self.actionbox.SetOutputPort("*","dummy")

		self.actionbox.Ready()
		self.actionbox.CeaseFire(self.frame)


