#=========================================================================================
#@(#)Thierry: $Id$
#
#    Handle the actionbox GUI and associated datas
#
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Thierry CHEVALIER <t_chevalier@libertysurf.fr>
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 

from Tkinter import *
from FileDialog import *
from tkSimpleDialog import *

import Pmw

import lib.utils
from lib.ServerDef import *
from lib.PortDef import *
from lib.ParameterDef import *
from lib.ActionDef import *
from lib.FactoryDef import *

from ActionBoxParameter import *

from GlobalVars import *
from LinkLine import *

from Action import *

from string import *
import re


class ActionBox:
	""" Handle the actionbox GUI and associated datas"""
	
	_bgColor= "#999999"		# Action background color
	_portBgColor= "#ffaa66"		# Port normal color
	_portFgColor= "#00ff00"		# Port highlight color
	
	# Generic methods
	#-------------------------------------
	def __init__(self, mapeditor, factorydef, actiondef, x, y):
		self.num= lib.utils.getuniqueid()

		self.factorydef= factorydef
		self.actiondef= actiondef
		self.mapeditor= mapeditor

		self.x= x
		self.y= y

		self.inports={}
		self.invalues={}
		self.inports["*"]=[]
		self.invalues["*"]=[]
		for p in self.actiondef.inputPorts.values():
			self.inports[p.name]=[]
			self.invalues[p.name]=[]

		self.outports={}
		self.outports["*"]=[]
		for p in self.actiondef.outputPorts.values():
			self.outports[p.name]=[]

		self._initGUI(mapeditor,actiondef)

		mapeditor.RegisterAction(self)

		self.actions={}

		if len(self.actiondef.inputPorts) == 0:
			self.FireReady()

	def __del__(self):
		try:
			self.mapeditor.UnregisterAction(self)
			for p in self.inports.values():
				for l in p:
					try:
						l.Destroy()
					except:
						pass
			for p in self.outports.values():
				for l in p:
					try:
						l.Destroy()
					except:
						pass
			self.wframe.destroy() 
			for p in self.actiondef.parameters.keys():
				self.parameters[p].set_local(p)
		except:
			pass
	
	def __repr__(self):
		return self.actiondef.name

	# Links related methods
	#-------------------------------------
	def RegisterInLink(self,link):
		print self.actiondef.type + " " + self.actiondef.name + " in linking on port " + link.PortnameOut
		self.inports[link.PortnameOut].append(link)
	def UnregisterInLink(self,link):
		self.inports[link.PortnameOut].remove(link)
	def RegisterOutLink(self,link):
		print self.actiondef.type + " " + self.actiondef.name + " out linking on port " + link.PortnameIn
		self.outports[link.PortnameIn].append(link)
	def UnregisterOutLink(self,link):
		self.outports[link.PortnameIn].remove(link)

	# status should be "on" or "off" to have any effect
	def SetInTriggerStatus(self,status):
		self.vinputtrigger.set(status)
		self._setinputtrigger()
	def GetInTriggerStatus(self):
		return self.vinputtrigger.get()

	def SetOutTriggerStatus(self,status):
		self.voutputtrigger.set(status)
	def GetOutTriggerStatus(self):
		return self.voutputtrigger.get()
	
	# Execution model
	#-------------------------------------
	def SetInputPort(self,portname,value):
		print self.actiondef.name,"> Received ",value," on input port",portname
		
		# A revoir <!!!>
		if self.vinputtrigger.get() == "off" and portname=="*": 
			print "    trigger not active, value >>>",value,"<<< discarded"
			return
		
		self.invalues[portname].append(value)
	
		print "Testing normal ports:"
		for portname in self.actiondef.inputorder:
			print "    port ",portname," holding ",str(len(self.invalues[portname]))," values"
			if len(self.invalues[portname]) == 0 :
				self.FireNotReady()
				return
		
		print "Testing trigger port (trigger status=",self.vinputtrigger.get(),"):"
		print "    trigger holding ",str(len(self.invalues["*"]))," values"
		if self.vinputtrigger.get() == "on" and len(self.invalues["*"]) == 0 :
			self.FireNotReady()
			return
		
		self.FireReady()
		
		if self.mapeditor.firemode == "auto":
			self.Fire()

	def Fire(self):
		print self.actiondef.name,"> Firing"

		for portname in self.actiondef.inputorder:
			if len(self.invalues[portname]) == 0 :
				print self.actiondef.name,"> ",portname," don't have any value, leaving..."
				self.FireNotReady()
				return
		if self.vinputtrigger.get() == "on" and len(self.invalues["*"]) == 0 :
			print self.actiondef.name,"> trigger don't have any value, leaving..."
			self.FireNotReady()
			return

		valuesin={}
		for portname in self.actiondef.inputorder:
			valuesin[portname]= self.invalues[portname][0]
			del self.invalues[portname][0]
			if len(self.invalues[portname]) == 0:
				self.FireNotReady()
		if self.vinputtrigger.get() == "on" : 
			del self.invalues["*"][0]
			if len(self.invalues["*"]) == 0:
				self.FireNotReady()

		parameters={}
		for p in self.actiondef.parameters.values():
			parameters[p.name]= self.parameters[p.name].get_value()
			
		serversname={}
		servershost={}
		serversport={}
		for s in self.actiondef.servers.values():
			serversname[s.type]= self.wserversname[s.type].get()
			servershost[s.type]= self.wservershost[s.type].get()
			serversport[s.type]= self.wserversport[s.type].get()
			
		valuesout={}
		for portname in self.actiondef.outputorder:
			valuesout[portname]= None

		if self.mapeditor.execmode == "execution":
			action= Action(self.mapeditor.factoriesIOR[self.factorydef.name],self.factorydef, self.actiondef, self)
			self.actions[action.frame]= action
			action.SetInput(valuesin)
			action.SetParameters(parameters)
			action.SetServers(serversname,servershost,serversport)
			action.start()
		else:
			print "================================================"
			print ">>> ",self.actiondef.name
			print "    inputs:"
			for name in self.actiondef.inputorder:
				print "            ",name,"=",valuesin[name]
			print "    parameters:"
			for name in self.actiondef.paramorder:
				print "            ",name,"=",parameters[name]
			print "    outputs:"
			for name in self.actiondef.outputorder:
				value= self.actiondef.name + "." + str(self.num) + "." + name
				print "            ",name,"=", value
				self.SetOutputPort(name,value)
			if self.voutputtrigger.get() == "on":
				print "            trigger"
				self.SetOutputPort("*","dummy")

	def SetOutputPort(self,portname,value):
		p= self.outports[portname]
		for l in p:
			l.Transmit(value)

	def CeaseFire(self,actionid):
		del self.actions[actionid]

	# External GUI piloting methods
	#-------------------------------------
	def FireReady(self):
		self.wfirebutton.configure(background="#00ff00")
		self.fireready=1
	def FireNotReady(self):
		self.wfirebutton.configure(background="#ff0000")
		self.fireready=-1

	def Ready(self):
		self.wname.configure(foreground="#000000")
		if self.fireready == 1:
			self.wfirebutton.configure(background="#00ff00")
		elif self.fireready == -1:
			self.wfirebutton.configure(background="#ff0000")
		else:
			self.wfirebutton.configure(background="#999999")
		self.wname.update()
	def Working(self):
		self.wfirebutton.configure(background="#ffff00")
		self.wname.configure(foreground="#ffff00")
		self.wname.update()

	def HighlightInputMenu(self):
		self.winput.configure(background=self._portFgColor)
	def NormalInputMenu(self):
		self.winput.configure(background=self._portBgColor)
	def HighlightOutputMenu(self):
		self.woutput.configure(background=self._portFgColor)
	def NormalOutputMenu(self):
		self.woutput.configure(background=self._portBgColor)

	def GetExpandState(self):
		return self._expand
	def SetExpandState(self,expand_state):
		if expand_state:
			self._doexpand()
		else:
			self._dounexpand()
	def Expand():
		self._doexpand()
	def UnExpand():
		self._dounexpand()
	
	# Internal GUI related methods
	#-------------------------------------
	def _select(self,e):
		self.lastx= e.x
		self.lasty= e.y
		self.wframe.tkraise()
		self.wname.grab_set()

	def _drag(self,e):
		self.x= self.x + (e.x - self.lastx)
		self.y= self.y + (e.y - self.lasty)
		self.mapeditor.wcanvas.coords(self.id, self.x, self.y)
		for p in self.inports.values():
			for l in p:
				l.Update()
		for p in self.outports.values():
			for l in p:
				l.Update()
		self.mapeditor.wscrolledcanvas.resizescrollregion()

	def _release(self,e):
		self.wname.grab_release()

	def _popinputmenu(self,e):
		self.lastx= e.x
		self.lasty= e.y
		self.winputpopup.post(e.x_root,e.y_root)
		self.winputpopup.grab_set()
		self.winputpopup.mainloop()

	def _popinputlistmenu(self,e):
		print "--- Current connected inputs ---"
		for pname in self.inports.keys():
			print pname,":"
			p= self.inports[pname]
			for l in p:
				print "\t from ",l.ActionBoxIn.actiondef.name,".",l.ActionBoxIn.num,".",l.PortnameIn
		
	def _popoutputmenu(self,e):
		self.lastx= e.x
		self.lasty= e.y
		self.woutputpopup.post(e.x_root,e.y_root)
		self.woutputpopup.grab_set()
		self.woutputpopup.mainloop()
		
	def _popoutputlistmenu(self,e):
		print "--- Current connected outputs ---"
		for pname in self.outports.keys():
			print pname,":"
			p= self.outports[pname]
			for l in p:
				print "\t to ",l.ActionBoxOut.actiondef.name,".",l.ActionBoxOut.num,".",l.PortnameOut

	def _toggleexpand(self):
		if self._expand:
			self._dounexpand()
		else:
			self._doexpand()
	def _doexpand(self):
		self.wsize.configure(text="o")
		self.wsframe.pack(side="top", fill="both", expand="yes", padx="4")
		self.wframe.tkraise()
		self._expand= 1
	def _dounexpand(self):
		self.wsize.configure(text="O")
		self.wsframe.pack_forget()
		self._expand= 0

	def _setinputtrigger(self):
		if self.vinputtrigger.get() == "on" and len(self.invalues["*"]) == 0 :
			self.FireNotReady()
		elif self.vinputtrigger.get() == "off":
			for portname in self.actiondef.inputorder:
				if len(self.invalues[portname]) == 0 :
					self.FireNotReady()
					return
			self.FireReady()
			
	def _initGUI(self,mapeditor,actiondef):
		# ActionBox frame
		self.wframe= Frame(mapeditor.wcanvas, background=self._bgColor, relief="raised")
		self.wframe.pack()
		self.balloon = Pmw.Balloon(self.wframe)

		# ActionBox upper toolbar
		self.wtb= Frame(self.wframe, relief="ridge")
		self.wtb.pack(side="top", fill="x", expand="yes", padx="4", pady="4")
		self.wname=Label(self.wtb, text=actiondef.name)
		self.wname.pack(side="left", padx="2", pady="2")
		self.wname.bind("<ButtonPress-1>", self._select)
		self.wname.bind("<B1-Motion>", self._drag)
		self.wname.bind("<ButtonRelease-1>", self._release)
		self.balloon.bind(self.wname, actiondef.desc)
		b= Button(self.wtb, text="X", command=self._destroy)
		self.balloon.bind(b, 'destroy action')
		b.pack(side="right", padx="2", pady="2")
		self.wfirebutton= Button(self.wtb, text=">", command=self.Fire)
		self.balloon.bind(self.wfirebutton, 'fire action')
		self.wfirebutton.pack(side="right", padx="2", pady="2")
		self.wsize= Button(self.wtb, text="O", command=self._toggleexpand)
		self.balloon.bind(self.wsize, 'expand parameters settings')
		self.wsize.pack(side="right", padx="2", pady="2")
		self._expand= 0

		# ActionBox parameter/servers/host widgets frame
		self.wsframe= Frame(self.wframe, relief="ridge")   

		self.parameters={}		# parameters
		for p in actiondef.parameters.values():
			self.parameters[p.name]= ActionBoxParameter(self,p,self.wsframe)

		f=Frame(self.wsframe, height="2", background="#000000")	# Just a separation line
		f.pack(side="top", fill="x", expand="yes")

		self.wserversname={}		# servers
		self.wservershost={}
		self.wserversport={}
		for s in actiondef.servers.values():
			self.wsname=Label(self.wsframe, text=s.type)
			self.wsname.pack(side="top")
			self.balloon.bind(self.wsname, s.desc)

			f=Frame(self.wsframe)
			f.pack(side="top", fill="x", expand="yes")

			Label(f, text="Name").pack(side="left")			# server name
			self.wserversname[s.type]=StringVar()
			e=Entry(f, textvariable=self.wserversname[s.type])
			e.pack(side="left", fill="x", expand="yes")
			self.balloon.bind(e, "Use name of a manually lauched server,\nor name of a registered server for dynamic activation")

			Label(f, text="Host").pack(side="left") 		# server execution hostname
			self.wservershost[s.type]=StringVar()
			e=Entry(f, textvariable=self.wservershost[s.type])
			e.pack(side="left", fill="x", expand="yes")
			self.balloon.bind(e, "Host on which your manual server is running,\nor host on which you want your dynamic server to be spawn")

			Label(f, text="Port").pack(side="left")			# server listening port
			self.wserversport[s.type]=StringVar()
			e=Entry(f, textvariable=self.wserversport[s.type])
			e.pack(side="left", fill="x", expand="yes")
			self.balloon.bind(e, "Port number of your manually started server,\n or use 1 for automatic activation")

		# ActionBox input/output ports frame
		ports= Frame(self.wframe,  background=self._bgColor) 
		ports.pack(side="bottom", fill="x", expand="yes", padx="2", pady="4")

		self.winput= Frame(ports, relief="raised", height="5", background=self._portBgColor)
		self.winput.pack(side="left", fill="x", expand="yes", padx="2")
		self.winput.bind("<ButtonPress-3>",self._popinputmenu)
		self.winputpopup=Menu(self.winput)
		self.vinputtrigger=StringVar()
		self.vinputtrigger.set("off")
		self.winputpopup.add_checkbutton(label="Use trigger",variable= self.vinputtrigger, onvalue="on", offvalue="off", command=self._setinputtrigger)
		self.winputpopup.add_command(label="Trigger",command= lambda mapeditor=mapeditor, self=self: mapeditor.AskForInput(self,"*"))
		for p in actiondef.inputPorts.values():
			self.winputpopup.add_command(label=p.name,command= lambda mapeditor=mapeditor, self=self, p=p : mapeditor.AskForInput(self,p.name))
		self.winput.bind("<ButtonPress-2>",self._popinputlistmenu)
		self.balloon.bind(self.winput, 'Input ports\nMouse middle key to print connection status\nMouse right key to connect')

		self.woutput= Frame(ports, relief="raised", height="5", background=self._portBgColor)
		self.woutput.pack(side="right", fill="x", expand="yes", padx="2")
		self.woutput.bind("<ButtonPress-3>",self._popoutputmenu)
		self.woutputpopup=Menu(self.woutput)
		self.voutputtrigger=StringVar()
		self.voutputtrigger.set("off")
		self.woutputpopup.add_checkbutton(label="Use trigger",variable= self.voutputtrigger, onvalue="on", offvalue="off")
		self.woutputpopup.add_command(label="Trigger",command=lambda mapeditor=mapeditor, self=self: mapeditor.AskForOutput(self,"*"))
		for p in actiondef.outputPorts.values():
			self.woutputpopup.add_command(label=p.name,command=lambda mapeditor=mapeditor, self=self, p=p : mapeditor.AskForOutput(self,p.name))
		self.woutput.bind("<ButtonPress-2>",self._popoutputlistmenu)
		self.balloon.bind(self.woutput, 'Output ports\nMouse middle key to print connection status\nMouse right key to connect')

		# Mapeditor canvas registration
		self.id=mapeditor.wcanvas.create_window(self.x, self.y, window=self.wframe, tag="Action")

	def _destroy(self):
		self.__del__()
