#=========================================================================================
#@(#)Thierry: $Id$
#
#    Handle the actionbox parameters
#
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Thierry CHEVALIER <t_chevalier@libertysurf.fr>
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 

from Tkinter import *
import Pmw

class ActionBoxParameter:
	"""Handle the actionbox parameters"""

	# Generic methods
	#-------------------------------------
	def __init__(self,actionbox,parameterdef,top):
		self.actionbox=actionbox
		self.parameterdef= parameterdef
		self.unsharedname= self.actionbox.actiondef.name + "." + str(self.actionbox.num) + "." + self.parameterdef.name
		self._initGUI(parameterdef,top)

	def __del__(self):
		if self.vstatus.get() == "globalshared": 
			self.actionbox.mapeditor.globalvars.unsubscribe(self.parameterdef.name) 
		elif self.vstatus.get() == "globalunshared":
			self.actionbox.mapeditor.globalvars.unsubscribe(self.unsharedname) 

	# Value external handling
	#-------------------------------------
	def set_value(self,value):
		self.vvalue.set(value)
	def get_value(self):
		return self.vvalue.get()
		
	# Status external handling
	#-------------------------------------
	def get_status(self):
		"""Return current status: globalshared, globalunshared or local"""
		return self.vstatus.get()
	def set_status(self,status):
		"""Set status. Should be one of: globalshared, globalunshared or local"""
		if status=="globalshared":
			self.set_globalshared()
		elif status=="globalunshared":
			self.set_globalunshared()
		elif status=="local":
			self.set_local()
		else:
			print "WARNING : in ActionBoxParameter setting unknown status >>>",status,"<<< to parameter ",self.unsharedname
	
	def set_globalshared(self):
		if self.oldstatus=="globalshared": return
		value=self.vvalue.get()
		if self.oldstatus == "globalunshared":
			self.actionbox.mapeditor.globalvars.unsubscribe(self.unsharedname) 
		self.vvalue=self.actionbox.mapeditor.globalvars.subscribe(self.parameterdef.name,value) 
		self.wvalue.configure(textvariable=self.vvalue)
		self.vstatus.set("globalshared")
		self.oldstatus="globalshared"
	def set_globalunshared(self):
		if self.oldstatus=="globalunshared": return
		value=self.vvalue.get()
		if self.oldstatus == "globalshared":
			self.actionbox.mapeditor.globalvars.unsubscribe(self.parameterdef.name) 
		self.vvalue=self.actionbox.mapeditor.globalvars.subscribe(self.unsharedname,value) 
		self.wvalue.configure(textvariable=self.vvalue)
		self.vstatus.set("globalunshared")
		self.oldstatus="globalunshared"
	def set_local(self):
		if self.oldstatus=="local": return
		value=self.vvalue.get()
		if self.oldstatus == "globalshared":
			self.actionbox.mapeditor.globalvars.unsubscribe(self.parameterdef.name) 
		elif self.oldstatus == "globalunshared":
			self.actionbox.mapeditor.globalvars.unsubscribe(self.unsharedname) 
		self.vvalue=StringVar()
		self.vvalue.set(value)
		self.wvalue.configure(textvariable=self.vvalue)
		self.vstatus.set("local")
		self.oldstatus="local"

	# Internal GUI methods
	#-------------------------------------
	def _initGUI(self,parameterdef,top):
		self.top= top

		self.frame=Frame(top)
		self.frame.pack(side="top", fill="x", expand="yes")
		self.balloon = Pmw.Balloon(self.frame)

		Label(self.frame,text=parameterdef.name).pack(side="left")

		self.vvalue=StringVar()
		self.wvalue=Entry(self.frame,textvariable=self.vvalue)
		self.balloon.bind(self.wvalue,parameterdef.desc)
		self.wvalue.pack(side="left", fill="x", expand="yes")

		self.vstatus=StringVar()
		self.vstatus.set("local")
		self.oldstatus="local"
		b=Radiobutton(self.frame, text="GS", value="globalshared", variable=self.vstatus, command=self.set_globalshared)
		self.balloon.bind(b, "Global Shared : Parameter value is global to this map editor,\nit's value will be shared with any same named parameter also posted global")
		b.pack(side="left")
		b=Radiobutton(self.frame, text="GU", value="globalunshared", variable=self.vstatus, command=self.set_globalunshared)
		self.balloon.bind(b, "Global Unshared : Parameter value appears in the global area of this map editor,\nit's value stay local to this action box" )
		b.pack(side="left")
		b=Radiobutton(self.frame, text="Local",  value="local",  variable=self.vstatus, command=self.set_local )
		self.balloon.bind(b, "Parameter value is local to this action box")
		b.pack(side="left")

