#=========================================================================================
#@(#)Thierry: $Id$
#
#    Handle factory investigation for dynamic action loading
#
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Thierry CHEVALIER <t_chevalier@libertysurf.fr>
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 

import sys, re

from Tkinter import *
import Pmw

from Fnorb.orb import CORBA,IOP

from lib.PortDef import *
from lib.ParameterDef import *
from lib.ActionDef import *
from lib.FactoryDef import *
from lib.NamingList import *

import _GlobalIDL

#==================================================
# Initialise the ORB
#--------------------------------------------------
orb = CORBA.ORB_init(sys.argv, CORBA.ORB_ID)


class FactoryLoader:
	"""Handle factory investigation for dynamic action loading"""
	
	def __init__(self, MapEditor):
		self.mapeditor= MapEditor
		self._initGUI(MapEditor.top)


	# Internal methods
	#----------------------------------------------
	def LoadFromNaming(self,name,obj):
		try:
			factory = obj._narrow(_GlobalIDL.FactoryAction)
		except:
			print "Unable to connect to this server:",str(obj)
			return
		(factoryname,)=re.match("^FactoryAction(.*)$",name).groups() 	
		self.Load(factoryname,factory)

	def LoadFromReffile(self,name,factoryref):
		global orb

		# Read the server's stringified IOR from a file (should use the naming service here)
		stringified_ior = open(factoryref).read()
		print "Factory stringified IOR=",stringified_ior

		# Convert the stringified IOR into an active object reference
		# We can't use direct 'factory =orb.string_to_object(stringified_ior)',
		# because we need to cast object in a FactoryActionstringified_ior
		# We use the following explicit sequence instead
		# Convert the stringified IOR into an IOR instance.
		ior= IOP.IOR()
		ior._fnorb_from_string(stringified_ior)
		# Create an object reference from the IOR, with appropriate casting to FactoryAction
		factory = apply(_GlobalIDL.FactoryAction, (ior,))
		self.Load(name,factory)

	def Load(self,factoryname,factory):
		global orb

		print "Factory ",factoryname," connected  !"
		# factory._FNORB_ID="IDL:FactoryAction:1.0"
		print factory._FNORB_ID


		f=FactoryDef(factoryname)

		actionlist= factory.getActions_list()
		for actionname in actionlist:
			print "\t Discovering action ",actionname

			a=ActionDef(actionname[6:])

			nbportin= factory.getNbInput(actionname)
			print "\t\t ",str(nbportin)," input ports"
			for i in range(1,nbportin+1):
				desc= factory.getInput_desc(actionname,i)
				type= factory.getInputType_desc(actionname,i)
				portname= factory.getInput_name(actionname,i)
				#portname= type + "_" + str(i)
				print "\t\t\t ",portname,"(",desc,")"
				a.AddInputPort(InputPortDef(portname,desc,type,"object"))

			nbportout= factory.getNbOutput(actionname)
			print "\t\t ",str(nbportout)," output ports"
			for i in range(1,nbportout+1):
				desc= factory.getOutput_desc(actionname,i)
				type= factory.getOutputType_desc(actionname,i)
				portname= factory.getOutput_name(actionname,i)
				#portname= type + "_" + str(i)
				print "\t\t\t ",portname,"(",desc,")"
				a.AddOutputPort(OutputPortDef(portname,desc,type,"object"))

			nbparam= factory.getNbParameter(actionname)
			print "\t\t ",str(nbparam)," parameters"
			for i in range(1,nbparam+1):
				desc= factory.getParameter_desc(actionname,i)
				type= factory.getParameterType_desc(actionname,i)
				paramname= factory.getParameter_name(actionname,i)
				#paramname= type + "_" + str(i)
				print "\t\t\t ",paramname,"(",desc,")"
				a.AddParameter(ParameterDef(paramname,desc,type))

			print "\t\t Adding server"
			a.AddServer(ServerDef("Server"))

			f.AddAction(a)
				
		print "Adding factory"
		self.mapeditor.factoriesDef[factoryname]= f
		self.mapeditor.factoriesIOR[factoryname]= orb.object_to_string(factory)

		print "Resetting menu"
		self.mapeditor._resetPopup()

		self.w.destroy()
		del self

	# Internal GUI methods
	#----------------------------------------------
	def _loadfromref(self):
		factoryname= self.name.get()
		factoryref= factoryname + ".ref"
		self.LoadFromReffile(factoryname,factoryref)

	def _dismiss(self):
		self.w.destroy()
		del self
		
	def _initGUI(self, top):
		self.w= Toplevel()
		self.balloon = Pmw.Balloon(self.w)

		self.wnb= Pmw.NoteBook(self.w, hull_width= 400, hull_height= 300)
		self.wnb.pack(side="top",fill="both", expand="yes")
		wfromnaming= self.wnb.add("Naming")
		wfromreffile= self.wnb.add("Reffile")

		f= Frame(wfromnaming)
		self.wnaming=NamingList(f,self.LoadFromNaming,"^FactoryAction")
		f.pack(side="top",fill="both",expand="yes")

		f= Frame(wfromreffile)
		Label(f, text="Ref file name", width="10").pack(side="top")
		self.name= StringVar()
		e=Entry(f, textvariable=self.name)
		e.pack(side="top",fill="x",expand="yes")
		self.balloon.bind(e, 'File containing an IOR string (.ref extension automatically added)')
		f.pack(side="top",fill="both",expand="yes")
		Button(wfromreffile, text="OK", command= self._loadfromref).pack(side="bottom")
		
		f= Frame(self.w)
		Button(f, text="Dismiss", command=self._dismiss).pack(side="right")
		f.pack(side="bottom", fill="x",expand="no")
		
		
