#=========================================================================================
#@(#)Thierry: $Id$
#
#    Handle the Group GUI and associated datas
#
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Thierry CHEVALIER <t_chevalier@libertysurf.fr>
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 

from Tkinter import *
from FileDialog import *
from tkSimpleDialog import *

import Pmw

import lib.utils
from lib.PortDef import *
from lib.ParameterDef import *
from lib.ActionDef import *
from lib.FactoryDef import *

from ActionBoxParameter import *

from FactoryLoader import *
		
from GlobalVars import *
from LinkLine import *

from ActionBox import *

from string import *
import re

class Group(ActionBox):
	"""Handle the Group GUI and associated datas"""
    
	def __init__(self, mapeditor, x, y, name="Group"):
		self.num= lib.utils.getuniqueid()
		
		self.mapeditor= mapeditor

		# Create a dediated actiondef
		self.actiondef= ActionDef(name,"group")

		# ActionBox related initializations
		self.x= x
		self.y= y

		self.inports={}
		self.inportsinternal={}
		self.invalues={}

		self.outports={}
		self.outportsinternal={}

		self._initGUI(mapeditor)

		self.mapeditor.RegisterAction(self)

	def __del__(self):
		ActionBox.__del__(self)

	def __repr__(self):
		return self.actiondef.name

	# Links related methods
	#-------------------------------------
	def RegisterInLink(self,link):
		print "Group register in link : link.PortnameOut=",link.PortnameOut
		ActionBox.RegisterInLink(self,link)
		(actionname,anum,portname,)=split(link.PortnameOut,".")
		num=atoi(anum)
		action= self.mymapeditor.actions[num]
		internallink= GroupInLink(self.mymapeditor,self,link.PortnameOut,action,portname)
		self.inportsinternal[link.PortnameOut]= internallink
	def UnregisterInLink(self,link):
		ActionBox.UnregisterInLink(self,link)
		del self.inportsinternal[link.PortnameOut]

	def RegisterOutLink(self,link):
		print "Group register out link : link.PortnameIn=",link.PortnameIn
		ActionBox.RegisterOutLink(self,link)
		(actionname,anum,portname)=split(link.PortnameIn,".")
		num=atoi(anum)
		action= self.mymapeditor.actions[num]
		internallink= GroupOutLink(self.mymapeditor,action,portname,self,link.PortnameIn)
		self.outportsinternal[link.PortnameIn]= internallink
	def UnregisterOutLink(self,link):
		ActionBox.UnregisterOutLink(self,link)
		del self.outportsinternal[link.PortnameIn]
		
	# Execution model
	#-------------------------------------
	def SetInputPort(self,portname,value):
		self.inportsinternal[portname].Transmit(value)

	def Fire(self):
		for a in self.mymapeditor.actions.values():
			a.Fire()

	# SetOutputPort is the same that ActionBox, no ?

	def CeaseFire(self,actionid):
		print "How did you managed to get there ??? (CeaseFire for a GROUP !)"


	# GUI related methods
	#-------------------------------------
	def _resetinputs(self):
		self.actiondef.CleanInputPorts()
		self.winputpopup=Menu(self.winput)
		for a in self.mymapeditor.actions.values():
			for p in a.actiondef.inputPorts.values():
				portname= "%s.%d.%s"%(a.actiondef.name, a.num, p.name)
				self.actiondef.AddInputPort(InputPortDef(portname,p.desc,p.type,p.category))
				self.inports[portname]=[]
				self.invalues[portname]=[]
				self.winputpopup.add_command(label=portname,command= lambda mapeditor=self.mapeditor, self=self, portname=portname : mapeditor.AskForInput(self,portname))
	def _popinputmenu(self,e):
		self.lastx= e.x
		self.lasty= e.y
		self._resetinputs()
		self.winputpopup.post(e.x_root,e.y_root)
		self.winputpopup.grab_set()
		self.winputpopup.mainloop()
		
	def _resetoutputs(self):
		self.actiondef.CleanOutputPorts()
		self.woutputpopup=Menu(self.woutput)
		for a in self.mymapeditor.actions.values():
			for p in a.actiondef.outputPorts.values():
				portname= "%s.%d.%s"%(a.actiondef.name, a.num, p.name)
				self.actiondef.AddOutputPort(OutputPortDef(portname,p.desc,p.type,p.category))
				self.outports[portname]=[]
				self.woutputpopup.add_command(label=portname,command= lambda mapeditor=self.mapeditor, self=self, portname=portname : mapeditor.AskForOutput(self,portname))
	def _popoutputmenu(self,e):
		self.lastx= e.x
		self.lasty= e.y
		self._resetoutputs()
		self.woutputpopup.post(e.x_root,e.y_root)
		self.woutputpopup.grab_set()
		self.woutputpopup.mainloop()
		
	def _rename(self,e):
		prompt= "Enter a new name for group "+self.vname.get()
		newname= askstring("New Group name", prompt)
		self.vname.set(newname)
		self.actiondef.name=newname

	def _initGUI(self,mapeditor):
		
		# Deals with the Group GUI in father map editor
		#----------------------------------------------
		# ActionBox frame
		self.wframe= Frame(mapeditor.wcanvas, background=self._bgColor, relief="raised")
		self.wframe.pack()
		self.balloon = Pmw.Balloon(self.wframe)

		# ActionBox upper toolbar
		self.wtb= Frame(self.wframe, relief="ridge")
		self.wtb.pack(side="top", fill="x", expand="yes", padx="4", pady="4")
		self.vname=StringVar()
		self.vname.set(self.actiondef.name)
		self.wname=Label(self.wtb, textvariable=self.vname)
		self.wname.bind("<ButtonPress-1>", self._select)
		self.wname.bind("<B1-Motion>", self._drag)
		self.wname.bind("<ButtonRelease-1>", self._release)
		self.balloon.bind(self.wname, 'use mouse right key to rename...')
		self.wname.pack(side="left", padx="2", pady="2")
		self.wname.bind("<ButtonPress-3>", self._rename)
		b= Button(self.wtb, text="X", command=self._destroy)
		self.balloon.bind(b, 'destroy group')
		b.pack(side="right", padx="2", pady="2")
		self.wfirebutton= Button(self.wtb, text=">", command=self.Fire)
		self.balloon.bind(self.wfirebutton, 'fire group, i.e. ask each of the group module to fire')
		self.wfirebutton.pack(side="right", padx="2", pady="2")
		self.wsize= Button(self.wtb, text="O", command=self._toggleexpand)
		self.balloon.bind(self.wsize, 'expand group for edition')
		self.wsize.pack(side="right", padx="2", pady="2")
		self._expand= 0

		# Internal MapEditor frame
		self.wsframe= Frame(self.wframe, relief="ridge")
		from MapEditor import *
		self.mymapeditor= MapEditor(self.wsframe)
		self.mymapeditor.factoriesDef=mapeditor.factoriesDef 
		self.mymapeditor._resetPopup()


		# ActionBox input/output ports frame
		ports= Frame(self.wframe,  background=self._bgColor)
		ports.pack(side="bottom", fill="x", expand="yes", padx="2", pady="4")

		self.winput= Frame(ports, relief="raised", height="5", background=self._portBgColor)
		self.winput.pack(side="left", fill="x", expand="yes", padx="2")
		self.winput.bind("<ButtonPress-3>",self._popinputmenu)

		self.woutput= Frame(ports, relief="raised", height="5", background=self._portBgColor)
		self.woutput.pack(side="right", fill="x", expand="yes", padx="2")
		self.woutput.bind("<ButtonPress-3>",self._popoutputmenu)

		# Mapeditor canvas registration
		self.id=mapeditor.wcanvas.create_window(self.x, self.y, window=self.wframe, tag="Group")	
