#=========================================================================================
#@(#)Thierry: $Id$
#
#    Handle the computationnal path visual building
#
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Thierry CHEVALIER <t_chevalier@libertysurf.fr>
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#	. retrofit load/save so that objects load/save themselves
#	. makes the canvas a separate class
#========================================================================================= 

from Tkinter import *
from FileDialog import *
from tkSimpleDialog import *
import Pmw

from lib.PortDef import *
from lib.ParameterDef import *
from lib.ActionDef import *
from lib.FactoryDef import *

from ActionBoxParameter import *
from FactoryLoader import *
from GlobalVars import *
from LinkLine import *
from Action import *
from ActionBox import *
from ParameterBox import *
from CommandBox import *
from Group import *

from string import *
import re

class MapEditor:
	"""Handle the computationnal path visual building"""

	# Generic methods
	#--------------------------------------------
	def __init__(self, top):
		self.top= top

		self.lastx= 0	# place where popup menu was invoked within window
		self.lasty= 0

		self.factoriesDef= {}
		self.factoriesIOR= {}

		self.actions={}
		self.links={}

		self.actionBoxIn= None
		self.portnameIn= None
		self.actionBoxOut= None
		self.portnameOut= None

		self.firemode= "manual"
		self.execmode= "execution"
		self.configloaded= None

		self._initGUI(top)

	# Inner objects registering
	#--------------------------------------------
	def RegisterAction(self,action):
		self.actions[action.num]= action
		self.wscrolledcanvas.resizescrollregion()
	def UnregisterAction(self,action):
		del self.actions[action.num]
		self.wscrolledcanvas.resizescrollregion()

	def RegisterLink(self,link):
		self.links[link.id]= link
	def UnregisterLink(self,link):
		del self.links[link.id]

	# Inner objects connection management
	#--------------------------------------------
	def AskForInput(self,actionbox,portname):
		self.actionBoxIn= actionbox
		self.portnameIn= portname
		if self.actionBoxOut is None:
			actionbox.HighlightInputMenu()
			print "Highlighting compatible output ports"
		else:
			self.Connect(self.actionBoxIn, self.portnameIn, self.actionBoxOut, self.portnameOut)
		
	def AskForOutput(self,actionbox,portname):
		self.actionBoxOut= actionbox
		self.portnameOut= portname
		if self.actionBoxIn is None:
			actionbox.HighlightOutputMenu()
			print "Highlighting compatible input ports"
		else:
			self.Connect(self.actionBoxIn, self.portnameIn, self.actionBoxOut, self.portnameOut)
		
	def Connect(self, actionBoxIn, portnameIn, actionBoxOut, portnameOut):

		if portnameIn == "*":
			print "Connecting ",actionBoxOut,".",portnameOut," => ",actionBoxIn,".",portnameIn
			LinkLine(self, actionBoxOut, portnameOut, actionBoxIn, portnameIn)
		elif actionBoxIn.actiondef.inputPorts[portnameIn].type == actionBoxOut.actiondef.outputPorts[portnameOut].type:
			print "Connecting ",actionBoxOut,".",portnameOut," => ",actionBoxIn,".",portnameIn
			LinkLine(self, actionBoxOut, portnameOut, actionBoxIn, portnameIn)
		else:
			print "Can't connect ",actionBoxOut,".",portnameOut," => ",actionBoxIn,".",portnameIn
			print actionBoxIn,".",portnameIn," is of type ",actionBoxIn.actiondef.inputPorts[portnameIn].type
			print actionBoxOut,".",portnameOut," is of type ",actionBoxOut.actiondef.outputPorts[portnameOut].type

		actionBoxIn.NormalInputMenu()
		self.actionBoxIn= None
		actionBoxOut.NormalOutputMenu()
		self.actionBoxOut= None

	# Load factory description
	#--------------------------------------------
	def LoadActionFactoryFromAF(self):
		loader= FactoryLoader(self)

	def LoadActionFactoryFromConfigFile(self):
		file= FileDialog(self.top).go()
		if file is None:
			print "Unable to open action factory specification"
		else:
			fid=open(file,'r')
			x= FactoryDefXMLParser()
			while 1:
				l= fid.readline()
				if l=="":
					break
				x.feed(l)
			fid.close()

			for factoryname in x.factories.keys():
				self.factoriesDef[factoryname]= x.factories[factoryname]

			x.close()
			del x

			self._resetPopup()
			self.configloaded= "true"
			self.wexecmode.config(background="#ff0000")


	# External GUI control
	#--------------------------------------------
	def ToggleFiremode(self):
		if self.firemode == "manual":
			self.wfiremode.config(text="Auto fire")
			self.firemode="auto"
		else:
			self.wfiremode.config(text="Manual fire")
			self.firemode="manual"
			
	def SetManualFiremode(self):
		self.wfiremode.config(text="Manual fire")
		self.firemode="manual"

	def ToggleExecmode(self):
		if self.execmode == "specification":
			if self.configloaded == None:
				self.wexecmode.config(text="Execution")
				self.execmode="execution"
				self.wAFLoadFromConfig.pack_forget()
			else:
				print "Unable to go back in Execution mode once AF.config loaded"
		else:
			self.wexecmode.config(text="Specification")
			self.execmode="specification"
			self.wAFLoadFromConfig.pack(side="left")


	# Internal GUI methods
	#--------------------------------------------

	# Canvas scrolling using mouse button1
	def _select(self,e):
		self.xdrag= e.x
		self.ydrag= e.y
	def _drag(self,e):
		dx= (e.x - self.xdrag)
		dy= (e.y - self.ydrag)
		self.wcanvas.xview_scroll(-dx, "unit")
		self.wcanvas.yview_scroll(-dy, "unit")
		self.xdrag= e.x
		self.ydrag= e.y
		self.wscrolledcanvas.resizescrollregion()

	# Popup menu using mouse button3
	def _popupmenu(self,e):
		self.lastx= e.x
		self.lasty= e.y 
		self.wpopup.post(e.x_root,e.y_root)
		self.wpopup.grab_set()
		self.wpopup.mainloop()

	def _resetPopup(self):
		self.wpopup=Menu(self.wcanvas)
		self.wpopup.add_command(label="New group",
								command= lambda self=self: 
												Group(self,self.lastx,self.lasty))
		self.wpopup.add_command(label="New parameter action",
								command= lambda self=self: 
												ParameterBox(self,self.lastx,self.lasty))
		self.wpopup.add_command(label="New command action",
								command= lambda self=self: 
												CommandBox(self,self.lastx,self.lasty))
		for factorydef in self.factoriesDef.values():
			m=Menu(self.wpopup)
			for actiondef in factorydef.actions.values():
				m.add_command(	label=actiondef.name,
								command= lambda self=self, 
											factorydef=factorydef, 
											actiondef=actiondef: 
												ActionBox(self,factorydef,actiondef,self.lastx,self.lasty))
			self.wpopup.add_cascade(label=factorydef.name, menu=m)

	def _generatepython(self):
		print "Automatic generation of a batch python code process controller not included in this release"

	def _generatecpp(self):
		print "Automatic generation of a batch C/C++ code process controller not included in this release"

	def _bigger(self):
		h=atoi(self.pane.interior().cget("height"))+10
		w=atoi(self.pane.interior().cget("width"))+10
		self.pane.interior().configure(height=str(h),width=str(w))
	def _smaller(self):
		h=atoi(self.pane.interior().cget("height"))-10
		w=atoi(self.pane.interior().cget("width"))-10
		self.pane.interior().configure(height=str(h),width=str(w))

	def _initGUI(self,top):

		# Toolbar
		f= Frame(top, relief="raised", height=25, background="#4444aa")
		f.pack(side="top", fill="x", expand="no")
		self.balloon = Pmw.Balloon(f)

		b=Button(f, bitmap="@bitmaps/OpenMap.xbm", command=self.OpenMap)
		self.balloon.bind(b, 'Load a map from file')
		b.pack(side="left", fill="y", expand="no")
		b=Button(f, bitmap="@bitmaps/SaveMap.xbm", command=self.SaveMap)
		self.balloon.bind(b, 'Save a map to file')
		b.pack(side="left", fill="y", expand="no")

		b=Button(f, bitmap="@bitmaps/OpenActionFactory.xbm", command=self.LoadActionFactoryFromAF)
		self.balloon.bind(b, 'Load actions from a running action factory')
		b.pack(side="left")

		self.wAFLoadFromConfig=Button(f, bitmap="@bitmaps/OpenActionFactory.xbm", command=self.LoadActionFactoryFromConfigFile)
		self.balloon.bind(self.wAFLoadFromConfig, 'Load actions from configuration files')

		self.wgeneratepython=Button(f, text=">>py", command=self._generatepython)
		self.balloon.bind(self.wgeneratepython, 'Generate a batch Python process controller,\ncorresponding to current map')
		self.wgeneratepython.pack(side="right")
		self.wgeneratecpp=Button(f, text=">>C++", command=self._generatecpp)
		self.balloon.bind(self.wgeneratecpp, 'Generate a batch C/C++ process controller,\ncorresponding to current map')
		self.wgeneratecpp.pack(side="right")

		self.wbigger=Button(f, text="+", command=self._bigger)
		self.balloon.bind(self.wbigger, 'Makes internal map editor window bigger')
		self.wbigger.pack(side="right")
		self.wsmaller=Button(f, text="-", command=self._smaller)
		self.balloon.bind(self.wsmaller, 'Makes internal map editor window smaller')
		self.wsmaller.pack(side="right")

		self.wfiremode=Button(f, text="Manual fire", command=self.ToggleFiremode)
		self.wfiremode.pack(side="right")

		self.wexecmode=Button(f, text="Execution", command=self.ToggleExecmode)
		self.wexecmode.pack(side="right")

		#  workarea
		self.pane = Pmw.PanedWidget(top, orient="horizontal")
		self.pane.add("global", min=50)
		self.pane.add("editor", min=100)

		# Global areas
		self.globalvars=GlobalVars(self,self.pane.pane("global"))

		# Canvas
		self.wscrolledcanvas=Pmw.ScrolledCanvas(self.pane.pane("editor"), borderframe=1)
		self.wcanvas=self.wscrolledcanvas.interior()
		self.wcanvas.configure(background="#888888", xscrollincrement="1", yscrollincrement="1")
		self.wcanvas.bind("<ButtonPress-3>",self._popupmenu)
		self.wcanvas.bind("<ButtonPress-1>",self._select)
		self.wcanvas.bind("<B1-Motion>",self._drag)
		self.wscrolledcanvas.pack(fill="both", expand="yes")

		self.pane.pack(side="top", fill="both", expand="yes")

		self._resetPopup()



	# Load/Save a map
	#--------------------------------------------
	def OpenMap(self):
		file= LoadFileDialog(self.top).go(pattern="*.map")
		if file is None:
			print "Unable to load map, no filename provided"
		else:
			print "=================== Map reading for ", file, " =================="
			# we could use an XML parser here...
			actions={}
			fid=open(file,'r')
			self._load(fid,self,actions)
			fid.close()
			print "=============== End of map reading for ", file, " ==============="

	def _load(self,fid,mapeditor,actions):
		while 1:
	 		l= fid.readline()

			if l=="":
				break
			elif string.find(l,"</GROUP") != -1:
				break;
			elif string.find(l,"<GROUP") != -1:
				# recover parameters for group creation
				(num,)=re.match("^[ ]*<GROUP[ ]*NUM[ ]*=[ ]*([0-9]*)[ ]*>[ ]*$",l).groups() 
				l= fid.readline()
				(groupname,)=re.match("^[ ]*<TYPE[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
				l= fid.readline()
				(rawx,rawy)=re.match("^[ ]*<POSITION[ ]*X=([\-0-9]*)[ ]*Y=([\-0-9]*)[ ]*/>[ ]*$",l).groups() 
				x= string.atoi(rawx)
				y= string.atoi(rawy)
				# create group and ask it recursively to load its part of the file
				actions[num]=Group(self,x,y,groupname)	
				actions[num].mymapeditor._load(fid,actions[num].mymapeditor,actions)
				actions[num]._resetinputs()
				actions[num]._resetoutputs()
			elif string.find(l,"<ACTION") != -1:
				# recover parameters for action creation
				(num,)=re.match("^[ ]*<ACTION[ ]*NUM[ ]*=[ ]*([0-9]*)[ ]*>[ ]*$",l).groups() 
				l= fid.readline()
				(actionname,)=re.match("^[ ]*<TYPE[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
				l= fid.readline()
				(rawx,rawy,rawexpandstate)=re.match("^[ ]*<POSITION[ ]*X=([\-0-9]*)[ ]*Y=([\-0-9]*)[ ]*EXPAND=([01])[ ]*/>[ ]*$",l).groups() 
				x= string.atoi(rawx)
				y= string.atoi(rawy)
				expandstate= string.atoi(rawexpandstate)
				l= fid.readline()
				(intriggerstatus,outtriggerstatus)=re.match("^[ ]*<TRIGGERSTATUS[ ]*IN[ ]*=[ ]*\"([^\"]*)\"[ ]*OUT[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
				l= fid.readline()
				parameters={}
				parameterstatus={}
				while string.find(l,"<PARAMETER") != -1:
					(name,value,status)=re.match("^[ ]*<PARAMETER[ ]*NAME[ ]*=[ ]*([^ ]*)[ ]*VALUE[ ]*=[ ]*\"([^ ]*)\"[ ]*STATUS[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups()
					parameters[name]= value
					parameterstatus[name]= status
					l= fid.readline()
				serversname={}
				servershost={}
				serversport={}
				while string.find(l,"<SERVER") != -1:
					(type,name,host,port)=re.match("^[ ]*<SERVER[ ]*TYPE[ ]*=[ ]*([^ ]*)[ ]*NAME[ ]*=[ ]*([^ ]*)[ ]*HOST[ ]*=[ ]*([^ ]*)[ ]*PORT[ ]*=[ ]*([^ ]*)[ ]*/>",l).groups()
					serversname[type]= name
					servershost[type]= host
					serversport[type]= port
					l= fid.readline()
				# check that this action description is available in the current available factories
				action=None
				for f in self.factoriesDef.values():
					for a in f.actions.values():
						if a.name==actionname:
							factory=f
							action=a
				# create action if found
				if action==None:
					print "Skipping action ",actionname," because no such action defined"
				else:
					actions[num]=ActionBox(mapeditor,factory,action,x,y)	
					print "Creating action ",actionname," ",num," with number >>>",actions[num].num,"<<< (expandstate= ",expandstate,")"
					actions[num].SetExpandState(expandstate)
					for pname in parameters.keys():
						print "\t parameter ",pname,"=",parameters[pname]," (",parameterstatus[pname],")"
						actions[num].parameters[pname].set_value(parameters[pname])
						actions[num].parameters[pname].set_status(parameterstatus[pname])
					actions[num].SetInTriggerStatus(intriggerstatus)
					actions[num].SetOutTriggerStatus(outtriggerstatus)
					for stype in serversname.keys():
						print "\t server ",stype," name=",serversname[stype]," host=",servershost[stype]," port=",serversport[stype]
						actions[num].wserversname[stype].set(serversname[stype])
						actions[num].wservershost[stype].set(servershost[stype])
						actions[num].wserversport[stype].set(serversport[stype])

			elif string.find(l,"<PARAMACTION") != -1:
				# recover parameters for paramaction creation
				(num,)=re.match("^[ ]*<PARAMACTION[ ]*NUM[ ]*=[ ]*([0-9]*)[ ]*>[ ]*$",l).groups() 
				l= fid.readline()
				(actionname,)=re.match("^[ ]*<TYPE[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
				l= fid.readline()
				(rawx,rawy,expandstate)=re.match("^[ ]*<POSITION[ ]*X=([\-0-9]*)[ ]*Y=([\-0-9]*)[ ]*EXPAND=([01])[ ]*/>[ ]*$",l).groups() 
				x= string.atoi(rawx)
				y= string.atoi(rawy)
				l= fid.readline()
				(order,)=re.match("^[ ]*<ORDER[ ]*VALUE[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups()
				
				# create paramaction
				actions[num]=ParameterBox(mapeditor,x,y)
				actions[num].ActionOrderSet(order)
				actions[num].Rename(actionname)
				actions[num].SetExpandState(expandstate)
				print "Creating paramaction ",actionname," ",num," with number >>>",actions[num].num,"<<<"
				
				
			elif string.find(l,"<COMMANDACTION") != -1:
				# recover parameters for paramaction creation
				(num,)=re.match("^[ ]*<COMMANDACTION[ ]*NUM[ ]*=[ ]*([0-9]*)[ ]*>[ ]*$",l).groups() 
				l= fid.readline()
				(actionname,)=re.match("^[ ]*<TYPE[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
				l= fid.readline()
				(rawx,rawy,expandstate)=re.match("^[ ]*<POSITION[ ]*X=([\-0-9]*)[ ]*Y=([\-0-9]*)[ ]*EXPAND=([01])[ ]*/>[ ]*$",l).groups() 
				x= string.atoi(rawx)
				y= string.atoi(rawy)
				l= fid.readline()
				(order,)=re.match("^[ ]*<ORDER[ ]*VALUE[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups()
				
				# create paramaction
				actions[num]=CommandBox(mapeditor,x,y)
				actions[num].ActionOrderSet(order)
				actions[num].Rename(actionname)
				actions[num].SetExpandState(expandstate)
				print "Creating commandaction ",actionname," ",num," with number >>>",actions[num].num,"<<<"
				
					
			elif string.find(l,"<LINK") != -1:
				# read link parameters
				l= fid.readline()
				l= fid.readline()  
				if string.find(l,"<GROUP") != -1:
					(actionnamein,numin)=re.match("^[ ]*<GROUP[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*NUM[ ]*=[ ]*([0-9]*)[ ]*/>[ ]*$",l).groups() 
					l= fid.readline()
					(portnamein,)=re.match("^[ ]*<PORT[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
					(aname,anum,portname)=re.match("([^.]*).([^.]*).(.*)$",portnamein).groups()
					portnamein= aname + "." + str(actions[anum].num) + "." + portname
				elif string.find(l,"<ACTION") != -1:
					(actionnamein,numin)=re.match("^[ ]*<ACTION[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*NUM[ ]*=[ ]*([0-9]*)[ ]*/>[ ]*$",l).groups() 
					l= fid.readline()
					(portnamein,)=re.match("^[ ]*<PORT[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
				elif string.find(l,"<PARAMACTION") != -1:
					(actionnamein,numin)=re.match("^[ ]*<PARAMACTION[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*NUM[ ]*=[ ]*([0-9]*)[ ]*/>[ ]*$",l).groups() 
					l= fid.readline()
					(portnamein,)=re.match("^[ ]*<PORT[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
				elif string.find(l,"<COMMANDACTION") != -1:
					(actionnamein,numin)=re.match("^[ ]*<COMMANDACTION[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*NUM[ ]*=[ ]*([0-9]*)[ ]*/>[ ]*$",l).groups() 
					l= fid.readline()
					(portnamein,)=re.match("^[ ]*<PORT[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
				l= fid.readline()
				l= fid.readline()
				l= fid.readline()
				if string.find(l,"<GROUP") != -1:
					(actionnameout,numout)=re.match("^[ ]*<GROUP[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*NUM[ ]*=[ ]*([0-9]*)[ ]*/>[ ]*$",l).groups() 
					l= fid.readline()
					(portnameout,)=re.match("^[ ]*<PORT[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
					(aname,anum,portname)=re.match("([^.]*).([^.]*).(.*)$",portnameout).groups()
					portnameout= aname + "." + str(actions[anum].num) + "." + portname
				elif string.find(l,"<ACTION") != -1:
					(actionnameout,numout)=re.match("^[ ]*<ACTION[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*NUM[ ]*=[ ]*([0-9]*)[ ]*/>[ ]*$",l).groups() 
					l= fid.readline()
					(portnameout,)=re.match("^[ ]*<PORT[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
				elif string.find(l,"<PARAMACTION") != -1:
					(actionnameout,numout)=re.match("^[ ]*<PARAMACTION[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*NUM[ ]*=[ ]*([0-9]*)[ ]*/>[ ]*$",l).groups() 
					l= fid.readline()
					(portnameout,)=re.match("^[ ]*<PORT[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
				elif string.find(l,"<COMMANDACTION") != -1:
					(actionnameout,numout)=re.match("^[ ]*<COMMANDACTION[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*NUM[ ]*=[ ]*([0-9]*)[ ]*/>[ ]*$",l).groups() 
					l= fid.readline()
					(portnameout,)=re.match("^[ ]*<PORT[ ]*NAME[ ]*=[ ]*\"([^\"]*)\"[ ]*/>[ ]*$",l).groups() 
				l= fid.readline()
				l= fid.readline()

				if not numin in actions.keys():
					print "Skipping link (",actionnamein,".",portnamein," => ",actionnameout,".",portnameout,")"
					print "  no ",actionnamein," action defined with number >>>",numin,"<<<"
				elif not numout in actions.keys():
					print "Skipping link (",actionnamein,".",portnamein," => ",actionnameout,".",portnameout,")"
					print "  no ",actionnameout," action defined with number >>>",numout,"<<<"
				else:
					print "Creating link(",actionnamein," ",numin,".",portnamein," => ",actionnameout," ",numout,".",portnameout,")"
					LinkLine(mapeditor, actions[numin], portnamein, actions[numout], portnameout)

			mapeditor.wscrolledcanvas.resizescrollregion()
					


	def SaveMap(self):
		file= SaveFileDialog(self.top).go(pattern="*.map")
		if file is None:
			print "Unable to save map, no filename provided"
		else:
			fid=open(file,'w')
			self._save(fid,"")
			fid.close()

	def _save(self,fid,indent):
			# save all actions/groups
			for a in self.actions.values():
				if a.actiondef.type=="group":
					s= "%s<GROUP NUM=%d>\n"%(indent,a.num)
					fid.write(s)
					s= "%s  <TYPE NAME=\"%s\"/>\n"%(indent,a.actiondef.name)
					fid.write(s)
					s= "%s  <POSITION X=%d Y=%d />\n"% (indent,a.x,a.y)
					fid.write(s)
					groupindent= indent + "  "
					a.mymapeditor._save(fid,groupindent)
					s= "%s</GROUP>\n"%(indent)
					fid.write(s)
				elif a.actiondef.type=="action":
					s= "%s<ACTION NUM=%d>\n"%(indent,a.num)
					fid.write(s)
					s= "%s  <TYPE NAME=\"%s\"/>\n"%(indent,a.actiondef.name)
					fid.write(s)
					s= "%s  <POSITION X=%d Y=%d EXPAND=%d/>\n"% (indent,a.x,a.y,a.GetExpandState())
					fid.write(s)
					s= "%s  <TRIGGERSTATUS IN=\"%s\" OUT=\"%s\" />\n"% (indent,a.GetInTriggerStatus(),a.GetOutTriggerStatus())
					fid.write(s)
					for p in a.actiondef.parameters.values():
						name= p.name
						value= a.parameters[p.name].get_value()
						status=a.parameters[p.name].get_status()
						s= "%s  <PARAMETER NAME=%s VALUE=\"%s\" STATUS=\"%s\"/>\n"%(indent,name,value,status)
						fid.write(s)
					for p in a.actiondef.servers.values():
						name= a.wserversname[p.type].get()
						host= a.wservershost[p.type].get()
						port= a.wserversport[p.type].get()
						s= "%s  <SERVER TYPE=%s NAME=%s HOST=%s PORT=%s />\n"%(indent,p.type,name,host,port)
						fid.write(s)
					s= "%s</ACTION>\n"%(indent)
					fid.write(s)
				elif a.actiondef.type=="paramaction":
					s= "%s<PARAMACTION NUM=%d>\n"%(indent,a.num)
					fid.write(s)
					s= "%s  <TYPE NAME=\"%s\"/>\n"%(indent,a.actiondef.name)
					fid.write(s)
					s= "%s  <POSITION X=%d Y=%d EXPAND=%d/>\n"% (indent,a.x,a.y,a.GetExpandState())
					fid.write(s)
					s= "%s  <ORDER VALUE=\"%s\"/>\n"%(indent,a.ActionOrderGet())
					fid.write(s)
					s= "%s</PARAMACTION>\n"%(indent)
					fid.write(s)
				elif a.actiondef.type=="commandaction":
					s= "%s<COMMANDACTION NUM=%d>\n"%(indent,a.num)
					fid.write(s)
					s= "%s  <TYPE NAME=\"%s\"/>\n"%(indent,a.actiondef.name)
					fid.write(s)
					s= "%s  <POSITION X=%d Y=%d EXPAND=%d/>\n"% (indent,a.x,a.y,a.GetExpandState())
					fid.write(s)
					s= "%s  <ORDER VALUE=\"%s\"/>\n"%(indent,a.ActionOrderGet())
					fid.write(s)
					s= "%s</COMMANDACTION>\n"%(indent)
					fid.write(s)
				
			# save all links
			for l in self.links.values():
				s= "%s<LINK NUM=%d>\n"%(indent,l.num)
				fid.write(s)
				s= "%s  <INPUT>\n"%(indent)
				fid.write(s)
				if l.ActionBoxIn.actiondef.type=="group":
					s= "%s    <GROUP NAME=\"%s\" NUM=%d />\n"%(indent,l.ActionBoxIn.actiondef.name, l.ActionBoxIn.num)
				elif l.ActionBoxIn.actiondef.type=="action":
					s= "%s    <ACTION NAME=\"%s\" NUM=%d />\n"%(indent,l.ActionBoxIn.actiondef.name, l.ActionBoxIn.num)
				elif l.ActionBoxIn.actiondef.type=="paramaction":
					s= "%s    <PARAMACTION NAME=\"%s\" NUM=%d />\n"%(indent,l.ActionBoxIn.actiondef.name, l.ActionBoxIn.num)
				elif l.ActionBoxIn.actiondef.type=="commandaction":
					s= "%s    <COMMANDACTION NAME=\"%s\" NUM=%d />\n"%(indent,l.ActionBoxIn.actiondef.name, l.ActionBoxIn.num)
				fid.write(s)
				s= indent+"    <PORT NAME=\"" + l.PortnameIn + "\"/>\n"
				fid.write(s)
				s= "%s  </INPUT>\n"%(indent)
				fid.write(s)
				s= "%s  <OUTPUT>\n"%(indent)
				fid.write(s)
				if l.ActionBoxOut.actiondef.type=="group":
					s= "%s    <GROUP NAME=\"%s\" NUM=%d />\n"%(indent,l.ActionBoxOut.actiondef.name, l.ActionBoxOut.num)
				elif l.ActionBoxOut.actiondef.type=="action":
					s= "%s    <ACTION NAME=\"%s\" NUM=%d />\n"%(indent,l.ActionBoxOut.actiondef.name, l.ActionBoxOut.num)
				elif l.ActionBoxOut.actiondef.type=="paramaction":
					s= "%s    <PARAMACTION NAME=\"%s\" NUM=%d />\n"%(indent,l.ActionBoxOut.actiondef.name, l.ActionBoxOut.num)
				elif l.ActionBoxOut.actiondef.type=="commandaction":
					s= "%s    <COMMANDACTION NAME=\"%s\" NUM=%d />\n"%(indent,l.ActionBoxOut.actiondef.name, l.ActionBoxOut.num)
				fid.write(s)
				s= indent+"    <PORT NAME=\"" + l.PortnameOut + "\"/>\n"
				fid.write(s)
				s= "%s  </OUTPUT>\n"%(indent)
				fid.write(s)
				s= "%s</LINK>\n"%(indent)
				fid.write(s)
