#=========================================================================================
#@(#)Thierry: $Id$
#
#    Handle direct parameter update on input trigger
#
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Thierry CHEVALIER <t_chevalier@libertysurf.fr>
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 

from Tkinter import *
from FileDialog import *
from tkSimpleDialog import *
import Pmw

import lib.utils
from lib.PortDef import *
from lib.ParameterDef import *
from lib.ActionDef import *

from GlobalVars import *
from LinkLine import *
from ActionBox import *

from string import *
import re

class ParameterBox(ActionBox):
	"""Handle direct parameter update on input trigger"""

	# Generic methods
	#-------------------------------------
	def __init__(self, mapeditor, x, y, name="ParamAction"):
		self.num= lib.utils.getuniqueid()

		self.mapeditor= mapeditor
		self.x= x
		self.y= y
		
		self.actiondef= ActionDef(name,"paramaction")
		self.inports={}
		self.invalues={}
		self.inports["*"]=[]
		self.invalues["*"]=[]
		self.outports={}
		self.outports["*"]=[]
		
		self._initGUI(mapeditor,self.actiondef)

		mapeditor.RegisterAction(self)

	# Parameter action external handling
	#-------------------------------------
	def ActionOrderSet(self,order):
		self.vparamaction.set(order)
	def ActionOrderGet(self):
		return self.vparamaction.get()
	
	# Trigger model
	#-------------------------------------
	def SetInputPort(self,portname,value):
		print self.actiondef.name,"> triggered on input port"
		self._trigger()
		self.mapeditor.top.update_idletasks()
		p= self.outports["*"]
		for l in p:
			l.Transmit(value)

	def Fire(self):
		print self.actiondef.name,"> Firing"
		self._trigger()
		self.mapeditor.top.update_idletasks()
		p= self.outports["*"]
		for l in p:
			l.Transmit("dummy")

	def ManualTrigger(self):
		print self.actiondef.name,"> manually triggered, no output flushed"
		self._trigger()

	def _trigger(self):
		order= self._buildorder()
		if order is not None:
			print order
			try:
				eval(order)
			except:
				print "Error: unable to execute your order properly"
			
	def _buildorder(self):
		order=self.vparamaction.get()
		print "buildorder: initial order is : ",order
		try:
			(leftorder,rightorder)=split(order,"=")
			print "buildorder: left  : ",leftorder
			print "buildorder: right : ",rightorder
			for globalvar in self.mapeditor.globalvars.vars.values():
				oldstring= globalvar.name
				print "   replacing ",oldstring
				newleftstring= "self.mapeditor.globalvars.vars['%s'].set"%globalvar.name
				newrightstring= "self.mapeditor.globalvars.vars['%s'].get()"%globalvar.name
				leftorder=replace(leftorder,oldstring,newleftstring)
				rightorder=replace(rightorder,oldstring,newrightstring)
				print "      left  : ",leftorder
				print "      right : ",rightorder
			return leftorder + "(" + rightorder + ")"
		except:
			print "Error : unable to translate your order"
			return None


	# External GUI control methods
	#-------------------------------------
	def Rename(self,newname):
		self.vname.set(newname)
		self.actiondef.name=newname

	# Internal GUI related methods
	#-------------------------------------
	def _rename(self,e):
		prompt= "Enter a new name for paramaction "+self.vname.get()
		newname= askstring("New ParamAction name", prompt)
		self.vname.set(newname)
		self.actiondef.name=newname

	def _initGUI(self,mapeditor,actiondef):
		
		# ActionBox frame
		self.wframe= Frame(mapeditor.wcanvas, background=self._bgColor, relief="raised")
		self.wframe.pack()
		self.wframe.bind("<ButtonPress-1>", self._select)
		self.wframe.bind("<B1-Motion>", self._drag)
		self.balloon = Pmw.Balloon(self.wframe)

		# ActionBox upper toolbar
		self.wtb= Frame(self.wframe, relief="ridge")
		self.wtb.pack(side="top", fill="x", expand="yes", padx="4", pady="4")
		self.vname=StringVar()
		self.vname.set(self.actiondef.name)
		self.wname=Label(self.wtb, textvariable=self.vname)
		self.wname.bind("<ButtonPress-1>", self._select)
		self.wname.bind("<B1-Motion>", self._drag)
		self.wname.bind("<ButtonRelease-1>", self._release)
		self.balloon.bind(self.wname, 'use mouse right key to rename...')
		self.wname.pack(side="left", padx="2", pady="2")
		self.wname.bind("<ButtonPress-3>", self._rename)
		b= Button(self.wtb, text="X", command=self._destroy)
		b.pack(side="right", padx="2", pady="2")
		self.wfirebutton= Button(self.wtb, text=">", command=self.Fire)
		self.wfirebutton.pack(side="right", padx="2", pady="2")
		self.wsize= Button(self.wtb, text="O", command=self._toggleexpand)
		self.wsize.pack(side="right", padx="2", pady="2")
		self._expand= 0

		# ActionBox parameter/servers/host widgets frame
		self.wsframe= Frame(self.wframe, relief="ridge")   
		
		Label(self.wsframe,text="Action").pack(side="left",expand="no")
		self.vparamaction=StringVar()
		self.wparamvalue=Entry(self.wsframe,textvariable=self.vparamaction)
		self.wparamvalue.pack(side="left", fill="x", expand="yes")
		self.balloon.bind(self.wparamvalue, 'be careful to use properly str() and atoi() functions whenever needed...')
		Button(self.wsframe,text="Manual trigger",command=self.ManualTrigger).pack(side="left", expand="no")

		f=Frame(self.wsframe, height="2", background="#000000")	# Just a separation line
		f.pack(side="top", fill="x", expand="yes")

		# ActionBox input/output ports frame
		ports= Frame(self.wframe,  background=self._bgColor) 
		ports.pack(side="bottom", fill="x", expand="yes", padx="2", pady="4")

		self.winput= Frame(ports, relief="raised", height="5", background=self._portBgColor)
		self.winput.pack(side="left", fill="x", expand="yes", padx="2")
		self.winput.bind("<ButtonPress-3>", self._connectin)

		self.woutput= Frame(ports, relief="raised", height="5", background=self._portBgColor)
		self.woutput.pack(side="right", fill="x", expand="yes", padx="2")
		self.woutput.bind("<ButtonPress-3>",self._connectout)

		# Mapeditor canvas registration
		self.id=mapeditor.wcanvas.create_window(self.x, self.y, window=self.wframe, tag="ParamAction")

	def _connectin(self,e):
		self.mapeditor.AskForInput(self,"*")

	def _connectout(self,e):
		self.mapeditor.AskForOutput(self,"*")

	def HighlightInputMenu(self):
		self.winput.configure(background=self._portFgColor)
	def NormalInputMenu(self):
		self.winput.configure(background=self._portBgColor)

	def _destroy(self):
		self.__del__()
