#=========================================================================================
#@(#)Thierry: $Id$ - ParamlistEditor
#
#    Handle global paramlist handling (single + sublists)
#
# (C) 2000 by Dassault Aviation <thierry.chevalier@dassault-aviation.fr>
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#	. retrofit load/save so that objects load/save themselves
#	. makes the canvas a separate class
#========================================================================================= 

from Tkinter import *
import Pmw
from FileDialog import *
from tkSimpleDialog import *

from string import *
import re

from ParamFile import *
from ParameterEditor import *

class ParamlistEditor :

	# Generic methods
	#--------------------------------------------
	def __init__(self, parameterlist, name, top, isroot):
		self.parameterlist=parameterlist
		self.name=name
		self.top= top
		self.isroot= isroot

		self.parameters= {}
		self.subparamlist= {}

		self._initGUI(top,isroot)

	# Public methods
	#--------------------------------------------
	def Delete(self):
		if (not self.isroot):
			for param in self.parameters.keys():
				self.parameters[param].Delete()
			for sublist in self.subparamlist.keys():
				self.subparamlist[sublist].Delete()
			self.paramfile.Delete()
			del self.paramfile
			self.wf.destroy()
			self.parameterlist.DelSublist(self.name)

	def NewParameter(self):
		name=  self.vnewparamname.get()
		value= self.vnewparamvalue.get()
		self.AddParameter(name,value)
	def AddParameter(self,name,value):
		if (not self.parameters.has_key(name)):
			self.parameters[name]= ParameterEditor(self,self.wparamlistf,name,value)
	def DelParameter(self,name):
		del self.parameters[name]

	def NewSublist(self):
		name=  self.vnewsublistname.get()
		self.AddSublist(name)
	def AddSublist(self,name):
		if (not self.subparamlist.has_key(name)):
			page= self.wsubparamlistnb.add(name)
			page.focus_set()
			self.subparamlist[name]= ParamlistEditor(self,name,page,0)
	def DelSublist(self,name):
		self.wsubparamlistnb.delete(name)
		del self.subparamlist[name]

	def AddSublistFromFile(self,name,fid):
		self.AddSublist(name)
		self.subparamlist[name].paramfile.Load(fid,name)
		self.wsubparamlistnb.setnaturalsize()
		self.wpane.setnaturalsize()

	def Save(self,fid):
		for param in self.parameters.keys():
			self.parameters[param].Save(fid)
		for sublist in self.subparamlist.keys():
			s= "%%%s\n"%(sublist)
			fid.write(s)
			self.subparamlist[sublist].Save(fid)
			s= "%%%s\n"%(sublist)
			fid.write(s)

	# Internal GUI methods
	#--------------------------------------------
	def _initGUI(self,top,isroot):
		self.wf= Frame(top, relief="raised")
		self.wf.pack(side="top", fill="both", expand="yes")

		f=Frame(self.wf, background="#6666ff")
		f.pack(side="top",fill="x", expand="no",padx="5",pady="5")
		# New parameter
		self.vnewparamname= StringVar()
		e=Entry(f,textvariable=self.vnewparamname)
		e.pack(side="left",fill="x",expand="no",pady="5")
		l=Label(f,text="=",background="#6666ff")
		l.pack(side="left",pady="5")
		self.vnewparamvalue= StringVar()
		e=Entry(f,textvariable=self.vnewparamvalue)
		e.pack(side="left",fill="x",expand="no",pady="5")
		b=Button(f,text="Add parameter",command=self.NewParameter)
		b.pack(side="left",pady="5")
		# New sublist
		self.vnewsublistname= StringVar()
		e=Entry(f,textvariable=self.vnewsublistname)
		e.pack(side="left",fill="x",expand="no",pady="5")
		b=Button(f,text="Add sublist",command=self.NewSublist)
		b.pack(side="left", fill="x", expand="no",pady="5")
		# Delete
		if (not isroot):
			b=Button(f,text="Delete",command=self.Delete)
			b.pack(side="right", fill="x", expand="yes",pady="5")

		# Associated file setting
		self.paramfile= ParamFile(self,self.wf,isroot)

		# Parameter edition area
		self.wpane= Pmw.PanedWidget(self.wf, hull_width=400, hull_height=100)
		self.wpane.pack(side="top",fill="both",expand="yes")
		# Internal parameter frame
		self.wpane.add('paramlist', min=10)
		self.wparamlistf= self.wpane.pane("paramlist")
		# Sub-paramlist frame
		self.wpane.add('subparamlist', min=10, size=10)
		self.wsubparamlistnb= Pmw.NoteBook(self.wpane.pane("subparamlist"))
		self.wsubparamlistnb.pack(side="top",fill="both", expand="yes")

