#=========================================================================================
#@(#)Thierry: $Id$
#
#    Allow selection among naming service
#
#-----------------------------------------------------------------------------------------
#                                                                         
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   (C) 2000 by Thierry CHEVALIER <t_chevalier@libertysurf.fr>
#
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 

import sys, os, re

from Tkinter import *
import Pmw

from Fnorb.orb import CORBA
from Fnorb.cos.naming import CosNaming, NamingContext, BindingIterator

#==================================================
# Initialise the ORB.
#--------------------------------------------------
orb = CORBA.ORB_init(sys.argv, CORBA.ORB_ID)

# get reference to Name Service
try:
	obj = orb.resolve_initial_references("NameService")
	root_context = obj._narrow(CosNaming.NamingContext)
	if root_context is None:
		print "No Name Server available !"
		NameServerAvailable= 0
	else:
		NameServerAvailable= 1
except:
	print "Error connecting to the Name Server !"
	NameServerAvailable= 0
	
	
	
class NamingList:
	def __init__(self, top, callback, pattern="", viewall=None):
		self.top=top
		if viewall:
			self.viewall=1
		else:
			self.viewall=0
		self.pattern=pattern
		self.callback=callback
		self.contexts = {}
		if NameServerAvailable:
			self._initGUI(top)
		else:
			self._noGUI(top)
			
	def Reload(self):
		self.viewall= self.vviewall.get()
		self.wflist.destroy()
		self.wflist=Frame(self.wsf.interior())
		self.wflist.pack(expand="yes", fill="both")
		self.contexts = {}
		self._connectlist(root_context,"")

	# Internal methods
	#----------------------------------------
	def _tostring(self, name):
		# convert a CORBA name to a string
		mystr = ''
		for nc in name:
			sstr = ''
			if nc.id != '':
				sstr = nc.id
			if nc.kind != '':
				sstr = sstr + '.' + nc.kind
			if nc.id == '' and nc.kind == '':
				sstr = '.'
			if mystr == '':
				mystr = sstr
			else:
				mystr = mystr + '/' + sstr
		return mystr
    
	def _ismarked(self, nc):
    	# check, if we have been here already
	    global orb
	    return self.contexts.has_key(orb.object_to_string(nc))

	def _mark(self, nc):
    	# remember this context
	    global orb
	    self.contexts[orb.object_to_string(nc)] = ""
    
	def _connect(self, name, obj):
		apply(self.callback,(name,obj,))

	def _connectlist(self, nc, indent = "\t"):

		if self._ismarked(nc):
			return
		self._mark(nc)

		(dummy, iterator) = nc.list(0)
		if iterator is None:
			return
	
		result = 1
		while result:
			(result, binding) = iterator.next_one()
			if result:
				name= self._tostring(binding.binding_name)
				mystr = '%s/%s' % (indent,name)
				if binding.binding_type == CosNaming.ncontext:
					if re.match(self.pattern,mystr) or self.viewall :
						Label(self.wflist,text=mystr,fg="#666666").pack(side="top",anchor="w", expand="no", padx="12")
					
					indent_ = indent + "/" + name
					obj = nc.resolve(binding.binding_name)
					sub_nc = obj._narrow(CosNaming.NamingContext)
					self._connectlist(sub_nc, indent_)

				else:
					if re.match(self.pattern,mystr):
						obj = nc.resolve(binding.binding_name)
						b=Button(self.wflist,text=mystr,command= lambda self=self, obj=obj, mystr=mystr: self._connect(mystr,obj))
						b.pack(side="top",anchor="w", expand="no")
					else:
						if re.match(self.pattern,name) or self.viewall :
							Label(self.wflist,text=mystr).pack(side="top",anchor="w", expand="no", padx="12")

	def _initGUI(self, top):
		global root_context
		self.w= Frame(top)
		self.wsf= Pmw.ScrolledFrame(self.w, labelpos = 'n', label_text = 'Naming service content',
									usehullsize = 1, 
            				    	)
		self.wflist=Frame(self.wsf.interior())
		self.wflist.pack(expand="yes", fill="both")
		self._connectlist(root_context,"")
		self.wsf.pack(side="top",fill="both",expand="yes")
		self.vviewall= IntVar()
		self.vviewall.set(self.viewall)
		
		f=Frame(self.w)
		Checkbutton(f, text="Viewall", variable=self.vviewall, onvalue="1", offvalue="0", command=self.Reload).pack(side="left", expand="no")
		Label(f,text=self.pattern, fg="#666666").pack(side="left",expand="no", anchor="center", fill="x")
		Button(f, text="Reload", command=self.Reload).pack(side="right", expand="no")
		f.pack(side="bottom",expand="no", fill="x")
		
		self.w.pack(side="top",fill="both",expand="yes")

	def _noGUI(self, top):
		Label(top,text="No naming service available !").pack()
