#=========================================================================================
#@(#)Thierry: $Id$ - ActionDef
#
#    A class to describe what's available within an action
#
# (C) 2000 by Dassault Aviation <thierry.chevalier@dassault-aviation.fr>
#-----------------------------------------------------------------------------------------
#$Log$
#========================================================================================= 

from xmllib import *

from PortDef import *
from ServerDef import *
from ParameterDef import *

class ActionDefParser:
	def __init__(self):
		self.actions={}

	def start_actiondef(self,attrs):
		try:
			self.a= ActionDef(attrs["name"],attrs["type"],attrs["desc"])
			self.actions[self.a.name]= self.a
		except:
			print "ActionDefParser : action malformed attributes ",attrs

	def start_inputportdef(self,attrs):
		try:
			self.a.AddInputPort(InputPortDef(attrs["name"],attrs["desc"],attrs["type"],attrs["category"]))
		except:
			print "ActionDefParser : input port malformed attributes ",attrs

	def start_outputportdef(self,attrs):
		try:
			self.a.AddOutputPort(OutputPortDef(attrs["name"],attrs["desc"],attrs["type"],attrs["category"]))
		except:
			print "ActionDefParser : output port malformed attributes ",attrs

	def start_parameterdef(self,attrs):
		try:
			self.a.AddParameter(ParameterDef(attrs["name"],attrs["desc"],attrs["type"]))
		except:
			print "ActionDefParser : parameter malformed attributes ",attrs

	def start_serverdef(self,attrs):
		try:
			self.a.AddServer(ServerDef(attrs["type"],attrs["desc"]))
		except:
			print "ActionDefParser : server malformed attributes ",attrs

class ActionDefXMLParser(ActionDefParser,XMLParser):
	def __init__(self):
		ActionDefParser.__init__(self)
		XMLParser.__init__(self)

class ActionDef:

	def __init__(self, name="dummy", type="action",desc=""):
		self.name= name
		self.type= type
		self.desc= desc
		self.inputorder=[]
		self.inputPorts={}
		self.outputorder=[]
		self.outputPorts={}
		self.paramorder=[]
		self.parameters={}
		self.servers={}
		
	def __repr__(self):
		return self.name

	def CleanInputPorts(self):
		self.inputorder=[]
		self.inputPorts={}
		
	def AddInputPort(self, inputport):
		self.inputorder.append(inputport.name)
		self.inputPorts[inputport.name]= inputport
		
	def CleanOutputPorts(self):
		self.outputorder=[]
		self.outputPorts={}

	def AddOutputPort(self, outputport):
		self.outputorder.append(outputport.name)
		self.outputPorts[outputport.name]= outputport
		
	def AddParameter(self, parameter):
		self.paramorder.append(parameter.name)
		self.parameters[parameter.name]= parameter

	def AddServer(self, server):
		self.servers[server.type]= server


	def GetObjecttypenameList(self):
		objecttypelist=[]
		for portname in self.inputPorts.keys():
			if not self.inputPorts[portname].type in objecttypelist:
				objecttypelist.append(self.inputPorts[portname].type)
		for portname in self.outputPorts.keys():
			if not self.outputPorts[portname].type in objecttypelist:
				objecttypelist.append(self.outputPorts[portname].type)
		return objecttypelist

	def GetServertypeList(self):
		serverlist=[]
		for servertype in self.servers.keys():
			if not servertype in serverlist:
				serverlist.append(servertype)
		return serverlist

	def Save(self,fid,indent):
		s= "%s<actiondef name=\"%s\" type=\"%s\" desc=\"%s\" >\n"%(indent,self.name,self.type,self.desc)   
		fid.write(s)
		newindent= indent + "  "
		for portname in self.inputorder:
			self.inputPorts[portname].Save(fid,newindent)
		for portname in self.outputorder:
			self.outputPorts[portname].Save(fid,newindent)
		for paramname in self.paramorder:
			self.parameters[paramname].Save(fid,newindent)
		for servertype in self.servers.keys():
			self.servers[servertype].Save(fid,newindent)
		s= "%s</actiondef>\n"%(indent)   
		fid.write(s)

