#=========================================================================================
#@(#)Thierry: $Id$ - Graph
#
#    Grapher based on Blt Graph, with PS print and linked scrollbars...
#
# (C) 2000 by Dassault Aviation <thierry.chevalier@dassault-aviation.fr>
#-----------------------------------------------------------------------------------------
#$Log$
#-----------------------------------------------------------------------------------------
# Do list:
#========================================================================================= 

import Tkinter
import Pmw
import string

from tkSimpleDialog import *

BLT_PS_PRINT="/Projet/JULIUS_2/VPE/lib/ps.tcl"
if not os.path.exists(BLT_PS_PRINT):
	print "BLT_PS_PRINT does not exist, no PS print available..."
	print "BLT_PS_PRINT =",BLT_PS_PRINT
else:
	BLT_PS_PRINT_LOADED=None
	
	
class Graph:

	def __init__(self, top, X, Ynames, Yvals):
	
		global BLT_PS_PRINT_LOADED,BLT_PS_PRINT
		if not BLT_PS_PRINT_LOADED:
			print "Loading BLT_PS_PRINT..."
			top.tk.call('source',BLT_PS_PRINT)
			BLT_PS_PRINT_LOADED= 1
			
		self.top= top
	
		self._numElements= len(Yvals)
		self._names= Ynames
		self._vector_x = X
		self._vector_y = Yvals
		
		self._initGUI(top)
		
	def destroy(self):
		self._w.destroy()

	def graphprint(self):
		self._wgraph.tk.call('PsDialog',self._wgraph._w)
		
	# Apply global settings to all curves in graph
	#----------------------------------------------------
	def _setelementtype(self, type):
		elements = self._wgraph.element_names()
		apply(self._wgraph.element_delete, elements)

		if type == 'none':
			return

		colorList = Pmw.Color.spectrum(self._numElements)
		for elem in range(self._numElements):
			if elem == 0:
				hue = None
			else:
				hue = (elem + 1.0) / self._numElements * 6.28318
			foreground = colorList[elem]
			background = Pmw.Color.changebrightness(self, foreground, 0.8)
			if type == 'mixed':
				if elem < self._numElements / 2:
					bar = 0
				else:
					bar = 1
			elif type == 'bars':
				bar = 1
			else:
				bar = 0
			if bar:
				self._wgraph.bar_create(
					self._names[elem],
					xdata=self._vector_x,
					ydata=self._vector_y[elem],
					foreground = foreground,
					background = background)
			else:
				self._wgraph.line_create(
					self._names[elem],
					linewidth = 1,
					xdata=self._vector_x,
					ydata=self._vector_y[elem],
					smooth = self.smooth,
					color = foreground)

	def _setbarmode(self, tag):
		self._wgraph.configure(barmode = tag)

	def _setsmooth(self, tag):
		self.smooth= tag
		for element in self._wgraph.element_show():
			if self._wgraph.element_type(element) == 'line':
				self._wgraph.element_configure(element, smooth = tag)

	def _setlinewidth(self, linewidth):
		self.linewidth= linewidth
		for element in self._wgraph.element_show():
			if self._wgraph.element_type(element) == 'line':
				self._wgraph.element_configure(element, linewidth = linewidth)
				
	def _setsymbolsize(self, markersize):
		self.markersize= markersize
		for element in self._wgraph.element_show():
			if self._wgraph.element_type(element) == 'line':
				self._wgraph.element_configure(element, pixels = markersize)
				
	def _setbufferelements(self, tag):
		self._wgraph.configure(bufferelements = (tag == 'buffered'))

	def _setsortelements(self, tag):
		element_list = list(self._wgraph.element_show())
		if len(element_list) > 1:
			if (tag == 'normal') == (element_list[-1] != 'var0'):
				element_list.reverse()
				self._wgraph.element_show(element_list)

	# Highlight legent when mouse over corresponding element
	#----------------------------------------------------------------
	def _legend_activate(self, e):
		e.widget.legend_activate(e.widget.element_get_current())
	def _legend_desactivate(self, e):
		e.widget.legend_deactivate(e.widget.element_get_current())
		
	# Manage graph popup menu
	#----------------------------------------------
	def _graphpopmenu(self,e):
		self.lastx= e.x
		self.lasty= e.y
		self._wgraphpopmenu.post(e.x_root,e.y_root)
		self._wgraphpopmenu.grab_set()
		self._wgraphpopmenu.mainloop()
	
	# Manage scrollbars / graph relationships
	#----------------------------------------------
	def _xbarset(self, v1,v2):
		self._xbar.set(v1,v2)
	def _xbarview(self, v1,v2):
		self._wgraph.axis_view_x(v1,v2)
	def _ybarset(self, v1,v2):
		self._ybar.set(v1,v2)
	def _ybarview(self, v1,v2):
		self._wgraph.axis_view_y(v1,v2)
		
	# Initialize GUI
	#------------------------
	def _initGUI(self, top):
		self._w= Tkinter.Frame(top)
		ff=Tkinter.Frame(self._w, relief="ridge")
		self._wgraph = Pmw.Blt.Stripchart(ff, plotrelief="sunken")
		self._wgraph.pack(anchor="center", expand="yes", fill="both")
		self._xbar	= Tkinter.Scrollbar(ff, command=self._xbarview, orient="horizontal")
		self._xbar.pack(side="bottom", anchor="n", expand="no", fill="x")
		ff.pack(side="left", expand="yes", fill="both")
		self._ybar	= Tkinter.Scrollbar(self._w, command=self._ybarview, orient="vertical")
		self._ybar.pack(side="right", anchor="w", expand="no", fill="y")
		self._w.pack(side="top", expand="yes", fill="both")
		
		self._wgraph.axis_configure("x", scrollcommand=self._xbarset, logscale="no", loose="no")
		self._wgraph.axis_configure("y", scrollcommand=self._ybarset, logscale="no")
		
		self._wgraph.grid_on()
		self._wgraph.zoom_stack()
		self._wgraph.crosshairs()
		self._wgraph.active_legend()
		self._wgraph.closest_point()
		self._wgraph.element_bind("all","<Enter>", self._legend_activate)
		self._wgraph.element_bind("all","<Leave>", self._legend_desactivate)
		
		self._setsmooth('quadratic')
		self._setelementtype('lines')
		self._setlinewidth(1)
		self._setsymbolsize(1)
		
		self._wgraph.bind("<ButtonPress-3>", self._graphpopmenu)
		self._wgraphpopmenu=Menu(self._wgraph)
		self._wgraphpopmenu.add_command(label="Print",command=self.graphprint)
